#include <QLabel>
#include <QRadioButton>
#include <QPushButton>
#include <QGridLayout>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QSignalMapper>
#include <QLabel>

#include "choosesequencedialog.h"
#include "phoneme.h"

ChooseSequenceDialog::ChooseSequenceDialog (QList< QList< QPointer<Phoneme> > > list,
                                            int initial, bool allowNone)  {
  sequenceList = list;
  freeMode = false;

  for (int x = 0; x < sequenceList.size (); x++)  {
    QString name = "";
    for (int y = 0; y < sequenceList[x].size (); y++)
      name += sequenceList[x][y]->getName ();

    sequenceButtonList.append (new QRadioButton (name));

    if (initial == x)
      sequenceButtonList[x]->setChecked (true);
  }

  if (allowNone)
    sequenceButtonList.append (new QRadioButton (QString::fromUtf8 ("∅")));

  int numCols = 20;
  if (sequenceButtonList.size () < 400) numCols -= 5;
  if (sequenceButtonList.size () < 225) numCols -= 5;
  if (sequenceButtonList.size () < 100) numCols -= 5;
  if (sequenceButtonList.size () < 25) numCols -= 1;
  if (sequenceButtonList.size () < 16) numCols -= 1;
  if (sequenceButtonList.size () < 9) numCols -= 1;

  sequenceLayout = new QGridLayout;

  for (int x = 0; x < sequenceButtonList.size (); x++)
    sequenceLayout->addWidget (sequenceButtonList[x], x/numCols, x % numCols);

  QLabel *chooseSequenceLabel = new QLabel ("Choose a sequence or phoneme:");
  submitButton = new QPushButton ("Done");
  submitButton->setMaximumSize (submitButton->sizeHint ());

  mainLayout = new QVBoxLayout;
  mainLayout->addWidget (chooseSequenceLabel);
  mainLayout->setAlignment (chooseSequenceLabel, Qt::AlignCenter);
  mainLayout->addLayout (sequenceLayout);
  mainLayout->addWidget (submitButton);
  mainLayout->setAlignment (submitButton, Qt::AlignCenter);

  setLayout (mainLayout);

  connect (submitButton, SIGNAL (clicked ()), this, SLOT (submit ()));
  connect (submitButton, SIGNAL (clicked ()), this, SLOT (hide ()));
}

ChooseSequenceDialog::ChooseSequenceDialog (QList< QPointer<Phoneme> > phonemeList,
                                            QList< QPointer<Phoneme> > sequence)  {
  sequenceList.append (phonemeList);
  currentSequence = sequence;
  freeMode = true;

  QSignalMapper *signalMapper = new QSignalMapper (this);
  for (int x = 0; x < sequenceList[0].size (); x++)  {
    phonemeButtonList.append (new QPushButton (sequenceList[0][x]->getName ()));
    connect (phonemeButtonList[x], SIGNAL (clicked ()), signalMapper, SLOT (map ()));
    signalMapper->setMapping (phonemeButtonList[x], sequenceList[0][x]);
  }

  int numCols = 20;
  if (phonemeButtonList.size () < 400) numCols -= 5;
  if (phonemeButtonList.size () < 225) numCols -= 5;
  if (phonemeButtonList.size () < 100) numCols -= 5;
  if (phonemeButtonList.size () < 25) numCols -= 1;
  if (phonemeButtonList.size () < 16) numCols -= 1;
  if (phonemeButtonList.size () < 9) numCols -= 1;

  sequenceLayout = new QGridLayout;

  for (int x = 0; x < phonemeButtonList.size (); x++)
    sequenceLayout->addWidget (phonemeButtonList[x], x/numCols, x % numCols);

  QLabel *chooseSequenceLabel = new QLabel ("Build a sequence or phoneme:");
  
  backButton = new QPushButton ("Back");
  backButton->setMaximumSize (backButton->sizeHint ());
  clearButton = new QPushButton ("Clear");
  clearButton->setMaximumSize (clearButton->sizeHint ());

  buttonLayout = new QHBoxLayout;
  buttonLayout->addWidget (backButton);
  buttonLayout->setAlignment (backButton, Qt::AlignRight);
  buttonLayout->addWidget (clearButton);
  buttonLayout->setAlignment (clearButton, Qt::AlignLeft);
  
  QString sequenceString = "";
  for (int x = 0; x < sequence.size (); x++)
    sequenceString += sequence[x]->getName ();

  sequenceDisplay = new QLabel (sequenceString);
  
  submitButton = new QPushButton ("Done");
  submitButton->setMaximumSize (submitButton->sizeHint ());

  mainLayout = new QVBoxLayout;
  mainLayout->addWidget (chooseSequenceLabel);
  mainLayout->setAlignment (chooseSequenceLabel, Qt::AlignCenter);
  mainLayout->addLayout (sequenceLayout);
  mainLayout->addLayout (buttonLayout);
  mainLayout->addWidget (sequenceDisplay);
  mainLayout->setAlignment (sequenceDisplay, Qt::AlignCenter);
  mainLayout->addWidget (submitButton);
  mainLayout->setAlignment (submitButton, Qt::AlignCenter);

  setLayout (mainLayout);

  connect (signalMapper, SIGNAL (mapped (QObject*)), this, SLOT (addPhoneme (QObject*)));
  connect (backButton, SIGNAL (clicked ()), this, SLOT (eraseLast ()));
  connect (clearButton, SIGNAL (clicked ()), this, SLOT (clear ()));
  connect (submitButton, SIGNAL (clicked ()), this, SLOT (submit ()));
  connect (submitButton, SIGNAL (clicked ()), this, SLOT (hide ()));
}

void ChooseSequenceDialog::addPhoneme (QObject *p)  {
  Phoneme *phoneme = qobject_cast<Phoneme*>(p);
  if (phoneme == NULL) return;
  
  currentSequence.append (phoneme);

  QString pstring = sequenceDisplay->text ();
  pstring += phoneme->getName ();
  sequenceDisplay->setText (pstring);
}

void ChooseSequenceDialog::eraseLast ()  {
  if (currentSequence.size () == 0)
    return;
  
  QPointer<Phoneme> phoneme = currentSequence.last ();

  QString pstring = sequenceDisplay->text ();
  pstring.chop (phoneme->getName ().size ());
  sequenceDisplay->setText (pstring);

  currentSequence.removeLast ();
}

void ChooseSequenceDialog::clear ()  {
  currentSequence.clear ();
  sequenceDisplay->setText ("");
}

void ChooseSequenceDialog::submit ()  {
  if (freeMode)  {
    emit sequenceChosen (currentSequence);
    return;
  }
  
  for (int x = 0; x < sequenceList.size (); x++)
    if (sequenceButtonList[x]->isChecked ())  {
      emit sequenceChosen (sequenceList[x]);
      return;
    }

  emit sequenceChosen (QList< QPointer<Phoneme> > ());
}