#ifndef DICTIONARY_H
#define DICTIONARY_H

#include <QTabWidget>
#include <QList>
#include <QPointer>
#include <QDomElement>

#include "syllable.h"

class Phoneme;

class QStringListModel;
class PhonologyPage;
class PhonotacticsPage;
class SuprasegmentalsPage;
class WordPage;
class SearchPage;
class DictionaryManagementPage;
class LoadPartialWordlistDialog;

class Dictionary : public QTabWidget {
  Q_OBJECT

  public:
    Dictionary ();
    void setPhonemeModel (QStringListModel*);
    void setWordModel (QStringListModel*);
    void setSuprasegmentalModel (QStringListModel*);

  signals:
    void languageNameUpdated (QString);

  private slots:
    void load (QString, int);
    void loadInventory (QDomElement);
    void loadPhonotactics (QDomElement);
    void loadSupras (QDomElement);
    void loadWordList (QDomElement);
    QList<Syllable> loadWordPhonology (QDomElement);

    void loadV0_1 (QString);
    void loadPartialWordlist (QDomElement);
    void loadText (QString, QString, bool);
    void loadLexique (QString, QStringList, QList<int>, bool);

    void save (QString);
    void saveText (QString, QString, bool);

    void setLanguageName (QString);

  private:
    void putInOrder (int*, int*, int*, int*);

    QString languageName;

    LoadPartialWordlistDialog *loadPartialDialog;

    SearchPage *searchPage;
    PhonologyPage *phonologyPage;
    PhonotacticsPage *phonotacticsPage;
    SuprasegmentalsPage *suprasegmentalsPage;
    WordPage *wordPage;
    DictionaryManagementPage *dictionaryManagementPage;
};

#endif
