#include <QString>
#include <QPushButton>
#include <QGroupBox>
#include <QComboBox>
#include <QLabel>
#include <QLineEdit>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QDir>
#include <QFileDialog>
#include <QMessageBox>
#include <QTextStream>
#include <QRegExp>
#include <QCheckBox>
#include <QDomDocument>
#include <QFile>
#include <QFont>
#include <QFontDialog>

#include <iostream>
using namespace std;

#include "const.h"
#include "dictionarymanagementpage.h"
#include "setparadigmsdialog.h"

DictionaryManagementPage::DictionaryManagementPage ()  {
  filename = "";
  
  QDir dir (".");
  dir.setNameFilters (QStringList ("*.cdic"));
  QStringList files = dir.entryList ();

  for (int x = 0; x < files.size (); x++)
    files[x].chop (5);

  dir.setNameFilters (QStringList () << "*.txt" << "*.csv");
  QStringList textFiles = dir.entryList ();

  paradigmName = QStringList () << "lx" << "ps" << "ge" << "de";
  paradigmUse = QList<int> () << 0 << 1 << 3 << 3;
  paradigmDialog = NULL;

  loadGroupBox = new QGroupBox ("Load a Dictionary");
  loadButton = new QPushButton ("Load");
  loadButton->setMaximumSize (loadButton->sizeHint ());
  loadButton->setMinimumSize (loadButton->sizeHint ());
  loadComboBox = new QComboBox;
  loadComboBox->addItems (files);
  loadComboBox->setEditable (false);
  loadComboBox->setMinimumSize (loadComboBox->sizeHint ());
  QLabel *fileFormatLabel = new QLabel ("File Format:");
  fileFormatLabel->setMinimumSize (fileFormatLabel->sizeHint ());
  fileFormatBox = new QComboBox;
  fileFormatBox->addItem ("Version 0.1");
  fileFormatBox->addItem ("Current Version (0.2)");
  fileFormatBox->setCurrentIndex (1);
  fileFormatBox->setMinimumSize (fileFormatBox->sizeHint ());

  topLoadLayout = new QHBoxLayout;
  topLoadLayout->addWidget (loadButton);
  topLoadLayout->addWidget (loadComboBox);
  bottomLoadLayout = new QHBoxLayout;
  bottomLoadLayout->addWidget (fileFormatLabel);
  bottomLoadLayout->addWidget (fileFormatBox);
  bottomLoadLayout->addStretch (1);
  loadLayout = new QVBoxLayout;
  loadLayout->addLayout (topLoadLayout);
  loadLayout->addLayout (bottomLoadLayout);
  loadGroupBox->setLayout (loadLayout);

  loadPartialGroupBox = new QGroupBox ("Import Partial Wordlist");
  loadPartialButton = new QPushButton ("Import");
  loadPartialButton->setMaximumSize (loadPartialButton->sizeHint ());
  loadPartialButton->setMinimumSize (loadPartialButton->sizeHint ());
  loadPartialBox = new QComboBox;
  loadPartialBox->addItems (files);
  loadPartialBox->setMinimumSize (loadPartialBox->sizeHint ());
  loadPartialLayout = new QHBoxLayout;
  loadPartialLayout->addWidget (loadPartialButton);
  loadPartialLayout->addWidget (loadPartialBox);
  loadPartialGroupBox->setLayout (loadPartialLayout);

  loadLexiqueGroupBox = new QGroupBox ("Import Wordlist from Lexique Pro");
  loadLexiqueBrowseButton = new QPushButton ("Browse...");
  loadLexiqueBrowseButton->setMaximumSize (loadLexiqueBrowseButton->sizeHint ());
  loadLexiqueBrowseButton->setMinimumSize (loadLexiqueBrowseButton->sizeHint ());
  loadLexiqueBrowseEdit = new QLineEdit;
  loadLexiqueBrowseEdit->setMinimumSize (loadLexiqueBrowseEdit->sizeHint ());
  loadLexiqueParaButton = new QPushButton ("Edit Paradigm Settings");
//  loadLexiqueParaButton->setMaximumSize (loadLexiqueParaButton->sizeHint ());
  loadLexiqueParaButton->setMinimumSize (loadLexiqueParaButton->sizeHint ());
//  loadLexiqueParaDisplay = new QLabel
//          ("<b>\\lx</b>: Word<br><b>\\ps</b>: Type<br><b>\\de</b>: Definition");
//  loadLexiqueParaDisplay->setMinimumSize (loadLexiqueParaDisplay->sizeHint ());
//  loadLexiqueParaDisplay->setAlignment (Qt::AlignCenter);
  loadLexiqueButton = new QPushButton ("Import");
  loadLexiqueButton->setMaximumSize (loadLexiqueButton->sizeHint ());
  loadLexiqueButton->setMinimumSize (loadLexiqueButton->sizeHint ());
  loadLexiqueOverwriteBox = new QCheckBox ("Overwrite identical words");

  loadLexiqueBrowseLayout = new QHBoxLayout;
  loadLexiqueBrowseLayout->addWidget (loadLexiqueBrowseButton);
  loadLexiqueBrowseLayout->addWidget (loadLexiqueBrowseEdit);
  loadLexiqueButtonLayout = new QHBoxLayout;
  loadLexiqueButtonLayout->addWidget (loadLexiqueParaButton);
  loadLexiqueButtonLayout->setAlignment (loadLexiqueParaButton, Qt::AlignRight);
  loadLexiqueButtonLayout->addWidget (loadLexiqueButton);
  loadLexiqueButtonLayout->setAlignment (loadLexiqueButton, Qt::AlignLeft);
  loadLexiqueLayout = new QVBoxLayout;
  loadLexiqueLayout->addLayout (loadLexiqueBrowseLayout);
//  loadLexiqueLayout->addWidget (loadLexiqueParaDisplay);
  loadLexiqueLayout->addLayout (loadLexiqueButtonLayout);
  loadLexiqueLayout->addWidget (loadLexiqueOverwriteBox);
  loadLexiqueLayout->setAlignment (loadLexiqueOverwriteBox, Qt::AlignCenter);
  loadLexiqueGroupBox->setLayout (loadLexiqueLayout);

  loadTextGroupBox = new QGroupBox ("Import Wordlist from Text or CSV File");
  loadTextBrowseButton = new QPushButton ("Browse...");
  loadTextBrowseButton->setMaximumSize (loadTextBrowseButton->sizeHint ());
  loadTextBrowseButton->setMinimumSize (loadTextBrowseButton->sizeHint ());
  loadTextBrowseEdit = new QLineEdit;
  loadTextBrowseEdit->setMinimumSize (loadTextBrowseEdit->sizeHint ());
  QLabel *loadFormulaLabel = new QLabel ("Formula:");
  loadTextFormulaEdit = new QLineEdit;
  loadTextFormulaEdit->setMinimumSize (loadTextFormulaEdit->sizeHint ());
  loadTextImportButton = new QPushButton ("Import");
  loadTextImportButton->setMaximumSize (loadTextImportButton->sizeHint ());
  loadTextImportButton->setMinimumSize (loadTextImportButton->sizeHint ());
  loadTextOverwriteBox = new QCheckBox ("Overwrite identical entries");

  loadTextBrowseLayout = new QHBoxLayout;
  loadTextBrowseLayout->addWidget (loadTextBrowseButton);
  loadTextBrowseLayout->addWidget (loadTextBrowseEdit);
  loadTextFormulaLayout = new QHBoxLayout;
  loadTextFormulaLayout->addWidget (loadFormulaLabel);
  loadTextFormulaLayout->addWidget (loadTextFormulaEdit);
  loadTextFormulaLayout->addWidget (loadTextImportButton);
  loadTextLayout = new QVBoxLayout;
  loadTextLayout->addLayout (loadTextBrowseLayout);
  loadTextLayout->addLayout (loadTextFormulaLayout);
  loadTextLayout->addWidget (loadTextOverwriteBox);
  loadTextGroupBox->setLayout (loadTextLayout);

  saveGroupBox = new QGroupBox ("Save the Current Dictionary");
  saveButton = new QPushButton ("Save As");
  saveButton->setMaximumSize (saveButton->sizeHint ());
  saveButton->setMinimumSize (saveButton->sizeHint ());
  saveComboBox = new QComboBox;
  saveComboBox->addItems (files);
  saveComboBox->setEditable (true);
  saveComboBox->setMinimumSize (saveComboBox->sizeHint ());
  saveLayout = new QHBoxLayout;
  saveLayout->addWidget (saveButton);
  saveLayout->addWidget (saveComboBox);
  saveGroupBox->setLayout (saveLayout);

  saveXHTMLGroupBox = new QGroupBox ("Export Wordlist to XHTML");
  saveXHTMLButton = new QPushButton ("Export As");
  saveXHTMLButton->setMaximumSize (saveXHTMLButton->sizeHint ());
  saveXHTMLButton->setMinimumSize (saveXHTMLButton->sizeHint ());
  saveXHTMLBox = new QComboBox;
//  saveXHTMLEntitiesButton = new QPushButton ("Edit Entities Settings");
//  saveXHTMLEntitiesButton->setMaximumSize (saveXHTMLEntitiesButton->sizeHint ());
//  saveXHTMLEntitiesDisplay = new QLabel;
  saveXHTMLTopLayout = new QHBoxLayout;
  saveXHTMLTopLayout->addWidget (saveXHTMLButton);
  saveXHTMLTopLayout->addWidget (saveXHTMLBox);
//  saveXHTMLLayout = new QVBoxLayout;
//  saveXHTMLLayout->addLayout (saveXHTMLTopLayout);
//  saveXHTMLLayout->addWidget (saveXHTMLEntitiesButton);
//  saveXHTMLLayout->setAlignment (saveXHTMLEntitiesButton, Qt::AlignCenter);
//  saveXHTMLLayout->addWidget (saveXHTMLEntitiesDisplay);
  saveXHTMLGroupBox->setLayout (saveXHTMLTopLayout);

  saveTextGroupBox = new QGroupBox ("Export Wordlist to Text or CSV File");
  saveTextButton = new QPushButton ("Export As");
  saveTextButton->setMaximumSize (saveTextButton->sizeHint ());
  saveTextButton->setMinimumSize (saveTextButton->sizeHint ());
  saveTextBox = new QComboBox;
  saveTextBox->addItems (textFiles);
  saveTextBox->setEditable (true);
  saveTextBox->setMinimumSize (saveTextBox->sizeHint ());
  QLabel *saveFormulaLabel = new QLabel ("Formula:");
  saveTextFormulaEdit = new QLineEdit;
  saveTextFormulaEdit->setMinimumSize (saveTextFormulaEdit->sizeHint ());
  saveTextOverwriteBox = new QCheckBox ("Overwrite file");

  saveTextTopLayout = new QHBoxLayout;
  saveTextTopLayout->addWidget (saveTextButton);
  saveTextTopLayout->addWidget (saveTextBox);
  saveTextFormulaLayout = new QHBoxLayout;
  saveTextFormulaLayout->addWidget (saveFormulaLabel);
  saveTextFormulaLayout->addWidget (saveTextFormulaEdit);
  saveTextLayout = new QVBoxLayout;
  saveTextLayout->addLayout (saveTextTopLayout);
  saveTextLayout->addLayout (saveTextFormulaLayout);
  saveTextLayout->addWidget (saveTextOverwriteBox);
  saveTextGroupBox->setLayout (saveTextLayout);

  inputLayout = new QVBoxLayout;
  inputLayout->addWidget (loadGroupBox);
  inputLayout->addWidget (loadPartialGroupBox);
  inputLayout->addWidget (loadLexiqueGroupBox);
  inputLayout->addWidget (loadTextGroupBox);
  inputLayout->addStretch (1);
  inputGroupBox = new QGroupBox ("Input");
  inputGroupBox->setLayout (inputLayout);

  outputLayout = new QVBoxLayout;
  outputLayout->addWidget (saveGroupBox);
  outputLayout->addWidget (saveXHTMLGroupBox);
  outputLayout->addWidget (saveTextGroupBox);
  outputLayout->addStretch (2);
  outputGroupBox = new QGroupBox ("Output");
  outputGroupBox->setLayout (outputLayout);

  centralLayout = new QHBoxLayout;
  centralLayout->addWidget (inputGroupBox);
  centralLayout->addWidget (outputGroupBox);

  languageNameButton = new QPushButton ("Set Language Name: ");
//  languageNameButton->setMaximumSize (languageNameButton->sizeHint ());
  languageNameButton->setMinimumSize (languageNameButton->sizeHint ());
  languageNameEdit = new QLineEdit;
  languageNameEdit->setMaximumSize (languageNameButton->sizeHint ());
  languageNameLayout = new QHBoxLayout;
  languageNameLayout->addWidget (languageNameButton);
  languageNameLayout->setAlignment (languageNameButton, Qt::AlignRight);
  languageNameLayout->addWidget (languageNameEdit);
  languageNameLayout->setAlignment (languageNameEdit, Qt::AlignLeft);

  setFontButton = new QPushButton ("Set Application Font");
//  setFontButton->setMaximumSize (setFontButton->sizeHint ());

  clearButton = new QPushButton ("Clear Current Dictionary");
//  clearButton->setMaximumSize (clearButton->sizeHint ());
  messageLabel = new QLabel ("");
  messageLabel->setAlignment (Qt::AlignCenter);

  mainLayout = new QVBoxLayout;
  mainLayout->addLayout (centralLayout);
  mainLayout->addLayout (languageNameLayout);
  mainLayout->addWidget (setFontButton);
  mainLayout->setAlignment (setFontButton, Qt::AlignCenter);
  mainLayout->addWidget (clearButton);
  mainLayout->setAlignment (clearButton, Qt::AlignCenter);
  mainLayout->addWidget (messageLabel);

  setLayout (mainLayout);

  connect (loadButton, SIGNAL (clicked ()), this, SLOT (load ()));
  connect (saveButton, SIGNAL (clicked ()), this, SLOT (save ()));
  connect (loadTextBrowseButton, SIGNAL (clicked ()), this, SLOT (browseText ()));
  connect (loadTextImportButton, SIGNAL (clicked ()), this, SLOT (loadText ()));
  connect (saveTextButton, SIGNAL (clicked ()), this, SLOT (saveText ()));
  connect (loadPartialButton, SIGNAL (clicked ()), this, SLOT (loadPartial ()));
  connect (loadLexiqueParaButton, SIGNAL (clicked ()), this, SLOT (launchParadigmDialog ()));
  connect (loadLexiqueBrowseButton, SIGNAL (clicked ()), this, SLOT (browseLexique ()));
  connect (loadLexiqueButton, SIGNAL (clicked ()), this, SLOT (loadLexique ()));
  connect (languageNameButton, SIGNAL (clicked ()), this, SLOT (setLanguageName ()));
  connect (setFontButton, SIGNAL (clicked ()), this, SLOT (setApplicationFont ()));
  connect (clearButton, SIGNAL (clicked ()), this, SIGNAL (clearLists ()));
  connect (clearButton, SIGNAL (clicked ()), messageLabel, SLOT (clear ()));
}

void DictionaryManagementPage::load ()  {
  if (loadComboBox->currentText () != "")  {
    emit loadFrom (loadComboBox->currentText (), fileFormatBox->currentIndex ());
    messageLabel->setText ("Loaded " + loadComboBox->currentText () + ".cdic.");
  }

  filename = loadComboBox->currentText ();
  refreshFileLists ();
}

void DictionaryManagementPage::save ()  {
  QString sname = saveComboBox->currentText ();
  
  if (sname == "")
    return;

  if (filename != "" && sname != filename)
    if (QMessageBox::question (this, "Confirmation", "Really overwrite " + sname + "?",
                               QMessageBox::Yes | QMessageBox::No) == QMessageBox::No)
      return;
  
  emit saveAs (saveComboBox->currentText ());
  messageLabel->setText ("Dictionary saved as " + saveComboBox->currentText ()
                           + ".cdic.");
  refreshFileLists ();
}

void DictionaryManagementPage::loadPartial ()  {
  if (loadPartialBox->currentText () == "") return;

  QFile file (loadPartialBox->currentText () + ".cdic");

  if (!file.open (QIODevice::ReadOnly))  {
    QMessageBox::warning (this, "",
                          "Could not open file.\nUse the text/CSV option to load from non-.cdic files.");
    return;
  }

  QDomDocument doc ("ConlangML");

  if (!doc.setContent (&file))  {
    QMessageBox::warning (this, "", "Could not read XML.");
    return;
  }

  file.close ();

  QDomElement wordList = doc.documentElement ().firstChildElement ("wordList");

  if (wordList.isNull ()) return;

  emit loadPartialWordlist (wordList);
}

void DictionaryManagementPage::browseText ()  {
  loadTextBrowseEdit->setText (
    QFileDialog::getOpenFileName (this, "Choose a text/CSV file to open", "."));
}

void DictionaryManagementPage::loadText ()  {
  // /w = word (text)
  // /t = type (text)
  // /s = subtype (text)
  // /d = definition (text)

  if (loadTextBrowseEdit->text () == "")  {
    QMessageBox::warning (this, "", "Please choose a file to load.");
    return;
  }

  if (loadTextFormulaEdit->text () == "")  {
    QMessageBox::warning (this, "", "Please enter a formula.");
    return;
  }

  if (!loadTextFormulaEdit->text ().contains ("/w"))  {
    QMessageBox::warning (this, "", "Your formula needs to have a \"/w\" in it, to match the word.");
    return;
  }

  if (loadTextFormulaEdit->text ().count ("/w") > 1 || loadTextFormulaEdit->text ().count ("/t") > 1 ||
      loadTextFormulaEdit->text ().count ("/s") > 1 || loadTextFormulaEdit->text ().count ("/d") > 1)  {
    QMessageBox::warning (this, "", "Please use only one of each matching symbol.");
    return;
  }

  QFile file (loadTextBrowseEdit->text ());

  if (!file.open (QIODevice::ReadOnly | QIODevice::Text))  {
    QMessageBox::warning (this, "", "Could not open file.");
    return;
  }

  QTextStream in (&file);
  QString line = "";
  QString formula = loadTextFormulaEdit->text ();
  formula.replace ("/w", ".+");
  formula.replace ("/t", ".+");
  formula.replace ("/s", ".+");
  formula.replace ("/d", ".+");
  formula.replace ("(", "\\(");
  formula.replace (")", "\\)");
  QRegExp regExp (formula);

  int matchedLines = 0;
  int totalLines = 0;

  while (!in.atEnd ())  {
    line = in.readLine ();

    if (regExp.exactMatch (line) && line != "")
      matchedLines++;

    totalLines++;
  };

  if (QMessageBox::warning (this, "", (QString)"Formula matched " +
                            QString::number (matchedLines) + (QString)" lines out of " +
                            QString::number (totalLines) + (QString)" lines.  Continue?",
                            QMessageBox::Yes, QMessageBox::No) == QMessageBox::No)
    return;

  emit loadTextFrom (loadTextBrowseEdit->text (), loadTextFormulaEdit->text (),
                     loadTextOverwriteBox->isChecked ());

  messageLabel->setText ("Wordlist loaded from " + loadTextBrowseEdit->text ());
}

void DictionaryManagementPage::saveText ()  {
  // /w = word
  // /t = type
  // /s = subtype
  // /p = phonology
  // /d = definition
  // Everything else = literal
/* durrr
  if (!saveTextFormulaEdit->text ().contains ("/w"))  {
    QMessageBox::warning (this, "", "You should at least include the word field in your save!");
    return;
  }
*/
  if (saveTextBox->currentText () != "")  {
    emit saveTextAs (saveTextBox->currentText (), saveTextFormulaEdit->text (),
                     saveTextOverwriteBox->isChecked ());
    messageLabel->setText ("Wordlist saved to " + saveTextBox->currentText ());
    refreshFileLists ();
  }
}

void DictionaryManagementPage::setLanguageName ()  {
  if (languageNameEdit->text () != "")
    emit setLanguageName (languageNameEdit->text ());
}

void DictionaryManagementPage::setApplicationFont ()  {
  bool ok = false;
  QFont font = QFontDialog::getFont (&ok, QFont ("DejaVu Sans", 8), this);

  if (ok)  {
    QFile settingsFile ("cd.config");
    QTextStream out (&settingsFile);

    if (settingsFile.open (QIODevice::WriteOnly | QIODevice::Text))  {
      out << "Font " << font.toString () << endl;
      settingsFile.close ();
      messageLabel->setText ("Font preference saved - restart CD to see effects.");
    }
  }
}

void DictionaryManagementPage::launchParadigmDialog ()  {
  if (paradigmDialog != NULL)
    delete paradigmDialog;

  paradigmDialog = new SetParadigmsDialog (paradigmName, paradigmUse);
  connect (paradigmDialog, SIGNAL (submit (QStringList, QList<int>)), this,
           SLOT (setParadigms (QStringList, QList<int>)));

  cout << "Not DMP's problem" << endl;
  paradigmDialog->show ();
}

void DictionaryManagementPage::setParadigms (QStringList paraList, QList<int> useList)  {
  paradigmName = paraList;
  paradigmUse = useList;
}

void DictionaryManagementPage::browseLexique ()  {
  loadLexiqueBrowseEdit->setText (
    QFileDialog::getOpenFileName (this, "Choose a db file to load", ".", "*.db"));
}

void DictionaryManagementPage::loadLexique ()  {
  QString fname = loadLexiqueBrowseEdit->text ();

  if (fname != "")  {
    emit loadLexiqueFrom (filename, paradigmName, paradigmUse, loadLexiqueOverwriteBox->isChecked ());
    messageLabel->setText ("Wordlist loaded from " + filename);
  }
}

void DictionaryManagementPage::refreshFileLists ()  {
  QDir dir (".");
  dir.setNameFilters (QStringList ("*.cdic"));
  QStringList files = dir.entryList ();

  for (int x = 0; x < files.size (); x++)
    files[x].chop (5);

  loadComboBox->clear ();
  loadComboBox->addItems (files);
  saveComboBox->clear ();
  saveComboBox->addItems (files);
  loadPartialBox->clear ();
  loadPartialBox->addItems (files);

  if (filename != "" && files.indexOf (filename) >= 0)
    saveComboBox->setCurrentIndex (files.indexOf (filename));

  dir.setNameFilters (QStringList () << "*.txt" << "*.csv");
  QStringList textFiles = dir.entryList ();

  saveTextBox->clear ();
  saveTextBox->addItems (textFiles);
}