#ifndef DICTIONARYMANAGEMENTPAGE_H
#define DICTIONARYMANAGEMENTPAGE_H

#include <QWidget>
#include <QDomElement>

class QString;
class QGroupBox;
class QPushButton;
class QComboBox;
class QLabel;
class QLineEdit;
class QCheckBox;
class QHBoxLayout;
class QVBoxLayout;
class QDomElement;
class SetParadigmsDialog;

class DictionaryManagementPage : public QWidget  {
  Q_OBJECT

  public:
    DictionaryManagementPage ();

  signals:
    void loadFrom (QString, int);
    void loadTextFrom (QString, QString, bool);
    void loadPartialWordlist (QDomElement);
    void saveAs (QString);
    void saveTextAs (QString, QString, bool);
    void loadLexiqueFrom (QString, QStringList, QList<int>, bool);
    void setLanguageName (QString);
    void clearLists ();

  private slots:
    void load ();
    void save ();
    void loadPartial ();
    void browseText ();
    void loadText ();
    void saveText ();
    
    void setLanguageName ();
    void setApplicationFont ();

    void launchParadigmDialog ();
    void setParadigms (QStringList, QList<int>);
    void browseLexique ();
    void loadLexique ();

  private:
    void refreshFileLists ();

    QString filename;
    QStringList paradigmName;
    QList<int> paradigmUse;

    SetParadigmsDialog *paradigmDialog;

    QGroupBox *inputGroupBox;
    QGroupBox *outputGroupBox;

    QVBoxLayout *inputLayout;
    QVBoxLayout *outputLayout;
    QHBoxLayout *centralLayout;

    QGroupBox *loadGroupBox;
    QPushButton *loadButton;
    QComboBox *loadComboBox;
    QComboBox *fileFormatBox;
    QHBoxLayout *topLoadLayout;
    QHBoxLayout *bottomLoadLayout;
    QVBoxLayout *loadLayout;

    QGroupBox *loadPartialGroupBox;
    QPushButton *loadPartialButton;
    QComboBox *loadPartialBox;
    QHBoxLayout *loadPartialLayout;

    QGroupBox *loadLexiqueGroupBox;
    QPushButton *loadLexiqueBrowseButton;
    QLineEdit *loadLexiqueBrowseEdit;
//    QLabel *loadLexiqueParaDisplay;
    QPushButton *loadLexiqueParaButton;
    QPushButton *loadLexiqueButton;
    QCheckBox *loadLexiqueOverwriteBox;
    QHBoxLayout *loadLexiqueBrowseLayout;
    QHBoxLayout *loadLexiqueButtonLayout;
    QVBoxLayout *loadLexiqueLayout;

    QGroupBox *loadTextGroupBox;
    QPushButton *loadTextBrowseButton;
    QLineEdit *loadTextBrowseEdit;
    QLineEdit *loadTextFormulaEdit;
    QPushButton *loadTextImportButton;
    QCheckBox *loadTextOverwriteBox;
    QHBoxLayout *loadTextBrowseLayout;
    QHBoxLayout *loadTextFormulaLayout;
    QVBoxLayout *loadTextLayout;

    QGroupBox *saveGroupBox;
    QPushButton *saveButton;
    QComboBox *saveComboBox;
    QHBoxLayout *saveLayout;

    QGroupBox *saveXHTMLGroupBox;
    QPushButton *saveXHTMLButton;
    QComboBox *saveXHTMLBox;
    QPushButton *saveXHTMLEntitiesButton;
    QLabel *saveXHTMLEntitiesDisplay;
    QHBoxLayout *saveXHTMLTopLayout;
    QVBoxLayout *saveXHTMLLayout;

    QGroupBox *saveTextGroupBox;
    QPushButton *saveTextButton;
    QComboBox *saveTextBox;
    QLineEdit *saveTextFormulaEdit;
    QCheckBox *saveTextOverwriteBox;
    QHBoxLayout *saveTextTopLayout;
    QHBoxLayout *saveTextFormulaLayout;
    QVBoxLayout *saveTextLayout;

    QPushButton *languageNameButton;
    QLineEdit *languageNameEdit;
    QHBoxLayout *languageNameLayout;
    QPushButton *setFontButton;

    QPushButton *clearButton;
    QLabel *messageLabel;

    QVBoxLayout *mainLayout;
};

#endif