#include <QComboBox>
#include <QPushButton>
#include <QString>
#include <QLabel>
#include <QGroupBox>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QSignalMapper>

#include <iostream>
using namespace std;

#include "editphonologydialog.h"
#include "syllable.h"
#include "phonotactics.h"
#include "suprasegmental.h"
#include "editsyllabledialog.h"
#include "const.h"

EditPhonologyDialog::EditPhonologyDialog (QString word, QList<Syllable> list,
                                          Phonotactics phonotax)  {
  syllableList = list;
  phonotactics = phonotax;
  editSyllableDialog = NULL;

  QSignalMapper *leftSignalMapper = new QSignalMapper (this);
  QSignalMapper *rightSignalMapper = new QSignalMapper (this);

  QList< QPointer<Suprasegmental> > allSupras = phonotactics.getSupraList ();
  for (int x = 0; x < allSupras.size (); x++)
    if (allSupras[x]->getDomain () == SYLL_APPL)  {
      if (allSupras[x]->getRepresentationType () == BEFORE)
        beforeSupras.append (allSupras[x]);
      else if (allSupras[x]->getRepresentationType () == FOLLOWING)
        followingSupras.append (allSupras[x]);
      else peakSupras.append (allSupras[x]);
    }

  QLabel *dialogLabel = new QLabel ("Edit the phonology of " + word + ":");

  addSyllableButton = new QPushButton ("Add Syllable");
  deleteSyllableButton = new QPushButton ("Delete Syllable:");
  deleteSyllableButton->setMaximumSize (deleteSyllableButton->sizeHint ());
  deleteComboBox = new QComboBox;
  for (int x = 0; x < syllableList.size (); x++)
    deleteComboBox->addItem (syllableList[x].getText ());
  deleteComboBox->setSizeAdjustPolicy (QComboBox::AdjustToContents);
  addDeleteLayout = new QHBoxLayout;
  addDeleteLayout->addWidget (addSyllableButton);
  addDeleteLayout->setAlignment (addSyllableButton, Qt::AlignRight);
  addDeleteLayout->addWidget (deleteSyllableButton);
  addDeleteLayout->addWidget (deleteComboBox);
  addDeleteLayout->setAlignment (deleteComboBox, Qt::AlignLeft);

  editSyllableButton = new QPushButton ("Edit Syllable:");
  editComboBox = new QComboBox;
  for (int x = 0; x < syllableList.size (); x++)
    editComboBox->addItem (syllableList[x].getText ());
  editComboBox->setSizeAdjustPolicy (QComboBox::AdjustToContents);
  editLayout = new QHBoxLayout;
  editLayout->addWidget (editSyllableButton);
  editLayout->setAlignment (editSyllableButton, Qt::AlignRight);
  editLayout->addWidget (editComboBox);
  editLayout->setAlignment (editComboBox, Qt::AlignLeft);

  wordLayout = new QHBoxLayout;

  for (int x = 0; x < syllableList.size (); x++)  {
    syllableBox.append (new QGroupBox ("Syllable " + QString::number (x + 1)));
    syllableLabel.append (new QLabel (syllableList[x].getText ()));

    peakSupraBox.append (new QComboBox);
    peakSupraBox[x]->addItem ("No Peak Supra");
    for (int y = 0; y < peakSupras.size (); y++)
      peakSupraBox[x]->addItem (peakSupras[y]->getName ());
    for (int y = 0; y < peakSupras.size (); y++)
      if (syllableList[x].hasSupra (peakSupras[y]))
        peakSupraBox[x]->setCurrentIndex (y + 1);

    beforeSupraBox.append (new QComboBox);
    beforeSupraBox[x]->addItem ("No Before Supra");
    for (int y = 0; y < beforeSupras.size (); y++)
      beforeSupraBox[x]->addItem (beforeSupras[y]->getName ());
    for (int y = 0; y < beforeSupras.size (); y++)
      if (syllableList[x].hasSupra (beforeSupras[y]))
        beforeSupraBox[x]->setCurrentIndex (y + 1);

    followingSupraBox.append (new QComboBox);
    followingSupraBox[x]->addItem ("No Following Supra");
    for (int y = 0; y < followingSupras.size (); y++)
      followingSupraBox[x]->addItem (followingSupras[y]->getName ());
    for (int y = 0; y < followingSupras.size (); y++)
      if (syllableList[x].hasSupra (followingSupras[y]))
        followingSupraBox[x]->setCurrentIndex (y + 1);

    leftButton.append (new QPushButton (QString::fromUtf8("«")));
    if (x == 0) leftButton[x]->setEnabled (false);
    rightButton.append (new QPushButton (QString::fromUtf8("»")));
    if (x == syllableList.size () - 1) rightButton[x]->setEnabled (false);
    moveLabel.append (new QLabel ("Move phonemes"));
    syllableButtonLayout.append (new QHBoxLayout);
    syllableButtonLayout[x]->addWidget (leftButton[x]);
    syllableButtonLayout[x]->addWidget (moveLabel[x]);
    syllableButtonLayout[x]->addWidget (rightButton[x]);

    connect (leftButton[x], SIGNAL (clicked ()), leftSignalMapper, SLOT (map ()));
    leftSignalMapper->setMapping (leftButton[x], x);
    connect (rightButton[x], SIGNAL (clicked ()), rightSignalMapper, SLOT (map ()));
    rightSignalMapper->setMapping (rightButton[x], x);

    syllableLayout.append (new QVBoxLayout);
    syllableLayout[x]->addWidget (syllableLabel[x]);
    syllableLayout[x]->setAlignment (syllableLabel[x], Qt::AlignCenter);
    syllableLayout[x]->addSpacing (20);
    syllableLayout[x]->addWidget (peakSupraBox[x]);
    syllableLayout[x]->addWidget (beforeSupraBox[x]);
    syllableLayout[x]->addWidget (followingSupraBox[x]);
    syllableLayout[x]->addLayout (syllableButtonLayout[x]);

    syllableBox[x]->setLayout (syllableLayout[x]);
    wordLayout->addWidget (syllableBox[x]);
  }

  submitButton = new QPushButton ("Submit");
  submitButton->setMaximumSize (submitButton->sizeHint ());

  mainLayout = new QVBoxLayout;
  mainLayout->addWidget (dialogLabel);
  mainLayout->addLayout (addDeleteLayout);
  mainLayout->addLayout (editLayout);
  mainLayout->addLayout (wordLayout);
  mainLayout->addWidget (submitButton);
  mainLayout->setAlignment (submitButton, Qt::AlignCenter);

  mainLayout->setSizeConstraint (QLayout::SetFixedSize);

  setLayout (mainLayout);

  connect (leftSignalMapper, SIGNAL (mapped (int)), this, SLOT (moveLeft (int)));
  connect (rightSignalMapper, SIGNAL (mapped (int)), this, SLOT (moveRight (int)));
  connect (addSyllableButton, SIGNAL (clicked ()), this, SLOT (addSyllable ()));
  connect (editSyllableButton, SIGNAL (clicked ()), this, SLOT (editSyllable ()));
  connect (deleteSyllableButton, SIGNAL (clicked ()), this, SLOT (deleteSyllable ()));
  connect (submitButton, SIGNAL (clicked ()), this, SLOT (submit ()));
  connect (submitButton, SIGNAL (clicked ()), this, SLOT (hide ()));
}

void EditPhonologyDialog::setPhonemeFont (QFont font)  {
  
}

void EditPhonologyDialog::moveLeft (int syllNum)  {
  if (syllNum <= 0 || syllNum >= syllableList.size ()) return;

  QPointer<Phoneme> movee;
  QList< QPointer<Suprasegmental> > moveeSupras;

  moveeSupras = syllableList[syllNum].popFirst (movee);
  syllableList[syllNum-1].appendToCoda (movee, moveeSupras);

  updateDisplay ();
}

void EditPhonologyDialog::moveRight (int syllNum)  {
  if (syllNum < 0 || syllNum >= syllableList.size () - 1) return;

  QPointer<Phoneme> movee;
  QList< QPointer<Suprasegmental> > moveeSupras;

  moveeSupras = syllableList[syllNum].popLast (movee);
  syllableList[syllNum+1].appendToOnset (movee, moveeSupras);

  updateDisplay ();
}

void EditPhonologyDialog::addSyllable ()  {
  syllableList.append (Syllable ());
  updateDisplay ();
}

void EditPhonologyDialog::editSyllable ()  {
  if (syllableList.size () == 0) return;

  if (editSyllableDialog) delete editSyllableDialog;

  int curr = editComboBox->currentIndex ();

  editSyllableDialog = new EditSyllableDialog (curr, syllableList[curr], phonotactics);

  connect (editSyllableDialog, SIGNAL (updateSyllable (int, Syllable)), this,
           SLOT (editSyllable (int, Syllable)));

  editSyllableDialog->exec ();
}

void EditPhonologyDialog::editSyllable (int num, Syllable syll)  {
  if (num >= syllableList.size () || num < 0) return;

  syllableList[num] = syll;
  updateDisplay ();
}

void EditPhonologyDialog::deleteSyllable ()  {
  if (syllableList.size () == 0) return;

  syllableList.removeAt (deleteComboBox->currentIndex ());
  updateDisplay ();
}

void EditPhonologyDialog::submit ()  {
  if (syllableList.size () != syllableBox.size ())
    updateDisplay ();

  for (int x = 0; x < syllableList.size (); x++)  {
    if (peakSupraBox[x]->currentIndex () == 0)
      syllableList[x].clearSupraOfType (DIACRITIC);
    else syllableList[x].addSupra (peakSupras[peakSupraBox[x]->currentIndex ()-1]);

    if (beforeSupraBox[x]->currentIndex () == 0)
      syllableList[x].clearSupraOfType (BEFORE);
    else syllableList[x].addSupra (beforeSupras[beforeSupraBox[x]->currentIndex ()-1]);

    if (followingSupraBox[x]->currentIndex () == 0)
      syllableList[x].clearSupraOfType (FOLLOWING);
    else syllableList[x].addSupra (followingSupras[followingSupraBox[x]->currentIndex ()-1]);
  }

  emit updatePhonology (syllableList);
}

void EditPhonologyDialog::updateDisplay ()  {
  while (syllableList.size () > syllableBox.size ())  {
    syllableBox.append (new QGroupBox ("Syllable " + QString::number (syllableBox.size () + 1)));
    syllableLabel.append (new QLabel (""));

    peakSupraBox.append (new QComboBox);
    peakSupraBox.last ()->addItem ("No Peak Supra");
    for (int y = 0; y < peakSupras.size (); y++)
      peakSupraBox.last ()->addItem (peakSupras[y]->getName ());

    beforeSupraBox.append (new QComboBox);
    beforeSupraBox.last ()->addItem ("No Before Supra");
    for (int y = 0; y < beforeSupras.size (); y++)
      beforeSupraBox.last ()->addItem (beforeSupras[y]->getName ());

    followingSupraBox.append (new QComboBox);
    followingSupraBox.last ()->addItem ("No Following Supra");
    for (int y = 0; y < followingSupras.size (); y++)
      followingSupraBox.last ()->addItem (followingSupras[y]->getName ());

    leftButton.append (new QPushButton (QString::fromUtf8 ("«")));
    if (leftButton.size () == 1) leftButton.last ()->setEnabled (false);
    if (rightButton.size () > 0) rightButton.last ()->setEnabled (true);
    rightButton.append (new QPushButton (QString::fromUtf8 ("»")));
    rightButton.last ()->setEnabled (false);
    moveLabel.append (new QLabel ("Move phoneme"));
    syllableButtonLayout.append (new QHBoxLayout);
    syllableButtonLayout.last ()->addWidget (leftButton.last ());
    syllableButtonLayout.last ()->addWidget (moveLabel.last ());
    syllableButtonLayout.last ()->addWidget (rightButton.last ());

    syllableLayout.append (new QVBoxLayout);
    syllableLayout.last ()->addWidget (syllableLabel.last ());
    syllableLayout.last ()->setAlignment (syllableLabel.last (), Qt::AlignCenter);
    syllableLayout.last ()->addSpacing (20);
    syllableLayout.last ()->addWidget (peakSupraBox.last ());
    syllableLayout.last ()->addWidget (beforeSupraBox.last ());
    syllableLayout.last ()->addWidget (followingSupraBox.last ());
    syllableLayout.last ()->addLayout (syllableButtonLayout.last ());

    syllableBox.last ()->setLayout (syllableLayout.last ());
    wordLayout->addWidget (syllableBox.last ());
  }

  while (syllableList.size () < syllableBox.size ())  {
    int last = syllableBox.size () - 1;

    QLabel *l = syllableLabel[last];
    syllableLayout[last]->removeWidget (l);
    syllableLabel.removeAt (last);
    delete l;

    QComboBox *c = peakSupraBox[last];
    syllableLayout[last]->removeWidget (c);
    peakSupraBox.removeAt (last);
    delete c;

    c = beforeSupraBox[last];
    syllableLayout[last]->removeWidget (c);
    beforeSupraBox.removeAt (last);
    delete c;

    c = followingSupraBox[last];
    syllableLayout[last]->removeWidget (c);
    followingSupraBox.removeAt (last);
    delete c;

    QPushButton *p = leftButton[last];
    syllableButtonLayout[last]->removeWidget (p);
    leftButton.removeAt (last);
    delete p;

    p = rightButton[last];
    syllableButtonLayout[last]->removeWidget (p);
    rightButton.removeAt (last);
    delete p;

    l = moveLabel[last];
    syllableButtonLayout[last]->removeWidget (l);
    moveLabel.removeAt (last);
    delete l;

    QHBoxLayout *blay = syllableButtonLayout[last];
    syllableLayout[last]->removeItem (syllableButtonLayout[last]);
    syllableButtonLayout.removeAt (last);
    delete blay;

    QVBoxLayout *lay = syllableLayout[last];
    syllableLayout.removeAt (last);
    delete lay;

    QGroupBox *g = syllableBox[last];
    wordLayout->removeWidget (g);
    syllableBox.removeAt (last);
    delete g;
  }

  deleteComboBox->clear ();
  editComboBox->clear ();

  for (int x = 0; x < syllableList.size (); x++)  {
    syllableLabel[x]->setText (syllableList[x].getText ());
    deleteComboBox->addItem (syllableList[x].getText ());
    editComboBox->addItem (syllableList[x].getText ());

    peakSupraBox[x]->setCurrentIndex (0);
    for (int y = 0; y < peakSupras.size (); y++)
      if (syllableList[x].hasSupra (peakSupras[y]))
        peakSupraBox[x]->setCurrentIndex (y + 1);

    beforeSupraBox[x]->setCurrentIndex (0);
    for (int y = 0; y < beforeSupras.size (); y++)
      if (syllableList[x].hasSupra (beforeSupras[y]))
        beforeSupraBox[x]->setCurrentIndex (y + 1);

    followingSupraBox[x]->setCurrentIndex (0);
    for (int y = 0; y < followingSupras.size (); y++)
      if (syllableList[x].hasSupra (followingSupras[y]))
        followingSupraBox[x]->setCurrentIndex (y + 1);
  }
}