#ifndef EDITPHONOLOGYDIALOG_H
#define EDITPHONOLOGYDIALOG_H

#include <QDialog>
#include <QList>
#include <QPointer>

#include "phonotactics.h"

class Syllable;
class Suprasegmental;
class EditSyllableDialog;
class QPushButton;
class QComboBox;
class QGroupBox;
class QLabel;
class QHBoxLayout;
class QVBoxLayout;

class EditPhonologyDialog : public QDialog  {
  Q_OBJECT

  public:
    EditPhonologyDialog (QString, QList<Syllable>, Phonotactics);
    void setPhonemeFont (QFont);

  signals:
    void updatePhonology (QList<Syllable>);

  private slots:
    void moveLeft (int);
    void moveRight (int);
    void addSyllable ();
    void editSyllable ();
    void editSyllable (int, Syllable);
    void deleteSyllable ();
    void submit ();

  private:
    void updateDisplay ();

    QList<Syllable> syllableList;
    Phonotactics phonotactics;
    QList< QPointer<Suprasegmental> > peakSupras;
    QList< QPointer<Suprasegmental> > beforeSupras;
    QList< QPointer<Suprasegmental> > followingSupras;

    EditSyllableDialog *editSyllableDialog;

    QPushButton *addSyllableButton;
    QPushButton *deleteSyllableButton;
    QComboBox *deleteComboBox;
    QPushButton *editSyllableButton;
    QComboBox *editComboBox;
    QPushButton *submitButton;

    QList<QGroupBox*> syllableBox;
    QList<QLabel*> syllableLabel;
    QList<QComboBox*> peakSupraBox;
    QList<QComboBox*> beforeSupraBox;
    QList<QComboBox*> followingSupraBox;
    QList<QPushButton*> leftButton;
    QList<QPushButton*> rightButton;
    QList<QLabel*> moveLabel;

    QHBoxLayout *addDeleteLayout;
    QHBoxLayout *editLayout;
    QList<QVBoxLayout*> syllableLayout;
    QList<QHBoxLayout*> syllableButtonLayout;
    QHBoxLayout *wordLayout;
    QVBoxLayout *mainLayout;
};

#endif