#include <QGroupBox>
#include <QLabel>
#include <QPushButton>
#include <QCheckBox>
#include <QGridLayout>
#include <QHBoxLayout>
#include <QVBoxLayout>

#include "editsyllabledialog.h"
#include "choosesequencedialog.h"
#include "syllable.h"
#include "phoneme.h"
#include "suprasegmental.h"
#include "phonotactics.h"
#include "const.h"

EditSyllableDialog::EditSyllableDialog (int num, Syllable syll, Phonotactics tax)  {
  syllableNum = num;
  syllable = syll;
  phonotactics = tax;
  chooseSequenceDialog = NULL;

  QList< QPointer<Suprasegmental> > supraList = phonotactics.getSupraList ();
  for (int x = 0; x < supraList.size (); x++)
    if (supraList[x]->getDomain () == PHON_APPL)
      supras.append (supraList[x]);

  onsetBox = new QGroupBox ("Onset");
  editOnsetButton = new QPushButton ("Edit Onset");
  editOnsetButton->setMaximumSize (editOnsetButton->sizeHint ());
  onsetRightButton = new QPushButton (QString::fromUtf8 ("»"));
  onsetLayout = new QGridLayout;

  QList< QPointer<Phoneme> > onset = syllable.getOnset ();
  onsetRightButton->setEnabled (onset.size () > 0);
  
  for (int x = 0; x < onset.size (); x++)  {
    onsetLabel.append (new QLabel (onset[x]->getName ()));

    onsetSupraBox.append (QList<QCheckBox*> ());

    for (int y = 0; y < supras.size (); y++)  {
      onsetSupraBox[x].append (new QCheckBox (supras[y]->getName ()));
      onsetSupraBox[x][y]->setChecked (syllable.onsetHasSupra (x, supras[y]));
      onsetSupraBox[x][y]->setEnabled (supras[y]->getPhonemeList ().contains (onset[x]));
    }

    onsetLayout->addWidget (onsetLabel[x], 1, x);
    onsetLayout->setAlignment (onsetLabel[x], Qt::AlignCenter);
    for (int y = 0; y < onsetSupraBox[x].size (); y++)
      onsetLayout->addWidget (onsetSupraBox[x][y], y + 2, x);
  }

  onsetLayout->addWidget (editOnsetButton, 0, 0, 1, onset.size ());
  onsetLayout->addWidget (onsetRightButton, supras.size () + 2,
                          (onset.size () > 0 ? onset.size () - 1 : 0));
  onsetBox->setLayout (onsetLayout);

  peakBox = new QGroupBox ("Peak");
  editPeakButton = new QPushButton ("Edit Peak");
  editPeakButton->setMaximumSize (editPeakButton->sizeHint ());
  peakLeftButton = new QPushButton (QString::fromUtf8 ("«"));
  peakRightButton = new QPushButton (QString::fromUtf8 ("»"));
  peakLayout = new QGridLayout;

  QList< QPointer<Phoneme> > peak = syllable.getPeak ();
  peakLeftButton->setEnabled (peak.size () > 0);
  peakRightButton->setEnabled (peak.size () > 0);
  
  for (int x = 0; x < peak.size (); x++)  {
    peakLabel.append (new QLabel (peak[x]->getName ()));

    peakSupraBox.append (QList<QCheckBox*> ());

    for (int y = 0; y < supras.size (); y++)  {
      peakSupraBox[x].append (new QCheckBox (supras[y]->getName ()));
      peakSupraBox[x][y]->setChecked (syllable.peakHasSupra (x, supras[y]));
      peakSupraBox[x][y]->setEnabled (supras[y]->getPhonemeList ().contains (peak[x]));
    }

    peakLayout->addWidget (peakLabel[x], 1, x);
    peakLayout->setAlignment (peakLabel[x], Qt::AlignCenter);
    for (int y = 0; y < peakSupraBox[x].size (); y++)
      peakLayout->addWidget (peakSupraBox[x][y], y + 2, x);
  }

  peakLayout->addWidget (editPeakButton, 0, 0, 1, peak.size ());
  peakLayout->addWidget (peakLeftButton, supras.size () + 2, 0);
  peakLayout->addWidget (peakRightButton, supras.size () + 2,
                         (peak.size () > 1 ? peak.size () - 1 : 1));
  peakBox->setLayout (peakLayout);

  codaBox = new QGroupBox ("Coda");
  editCodaButton = new QPushButton ("Edit Coda");
  editCodaButton->setMaximumSize (editCodaButton->sizeHint ());
  codaLeftButton = new QPushButton (QString::fromUtf8 ("«"));
  codaLayout = new QGridLayout;

  QList< QPointer<Phoneme> > coda = syllable.getCoda ();
  codaLeftButton->setEnabled (coda.size () > 0);
  
  for (int x = 0; x < coda.size (); x++)  {
    codaLabel.append (new QLabel (coda[x]->getName ()));

    codaSupraBox.append (QList<QCheckBox*> ());

    for (int y = 0; y < supras.size (); y++)  {
      codaSupraBox[x].append (new QCheckBox (supras[y]->getName ()));
      codaSupraBox[x][y]->setChecked (syllable.codaHasSupra (x, supras[y]));
      codaSupraBox[x][y]->setEnabled (supras[y]->getPhonemeList ().contains (coda[x]));
    }

    codaLayout->addWidget (codaLabel[x], 1, x);
    codaLayout->setAlignment (codaLabel[x], Qt::AlignCenter);
    for (int y = 0; y < codaSupraBox[x].size (); y++)
      codaLayout->addWidget (codaSupraBox[x][y], y + 2, x);
  }

  codaLayout->addWidget (editCodaButton, 0, 0, 1, coda.size ());
  codaLayout->addWidget (codaLeftButton, supras.size () + 2, 0);
  codaBox->setLayout (codaLayout);

  syllableLayout = new QHBoxLayout;
  syllableLayout->addWidget (onsetBox);
  syllableLayout->addWidget (peakBox);
  syllableLayout->addWidget (codaBox);

  submitButton = new QPushButton ("Submit");
  submitButton->setMaximumSize (submitButton->sizeHint ());

  mainLayout = new QVBoxLayout;
  mainLayout->addLayout (syllableLayout);
  mainLayout->addWidget (submitButton);
  mainLayout->setAlignment (submitButton, Qt::AlignCenter);

  mainLayout->setSizeConstraint (QLayout::SetFixedSize);

  setLayout (mainLayout);

  connect (editOnsetButton, SIGNAL (clicked ()), this, SLOT (chooseOnset ()));
  connect (editPeakButton, SIGNAL (clicked ()), this, SLOT (choosePeak ()));
  connect (editCodaButton, SIGNAL (clicked ()), this, SLOT (chooseCoda ()));

  connect (onsetRightButton, SIGNAL (clicked ()), this, SLOT (moveOnsetRight ()));
  connect (peakLeftButton, SIGNAL (clicked ()), this, SLOT (movePeakLeft ()));
  connect (peakRightButton, SIGNAL (clicked ()), this, SLOT (movePeakRight ()));
  connect (codaLeftButton, SIGNAL (clicked ()), this, SLOT (moveCodaLeft ()));
  
  connect (submitButton, SIGNAL (clicked ()), this, SLOT (submit ()));
  connect (submitButton, SIGNAL (clicked ()), this, SLOT (hide ()));
}

void EditSyllableDialog::chooseOnset ()  {
  if (chooseSequenceDialog) delete chooseSequenceDialog;

  if (phonotactics.phonotacticsInUse ())  {
    QList< QList< QPointer<Phoneme> > > list = phonotactics.getOnsetList ();
    chooseSequenceDialog = new ChooseSequenceDialog (list, list.indexOf (syllable.getOnset ()),
                                                     !phonotactics.onsetIsRequired ());
  }

  else
    chooseSequenceDialog = new ChooseSequenceDialog (phonotactics.getPhonemeList (),
                                                     syllable.getOnset ());

  connect (chooseSequenceDialog, SIGNAL (sequenceChosen (QList< QPointer<Phoneme> >)),
           this, SLOT (setOnset (QList< QPointer<Phoneme> >)));

  chooseSequenceDialog->exec ();
}

void EditSyllableDialog::choosePeak ()  {
  if (chooseSequenceDialog) delete chooseSequenceDialog;

  if (phonotactics.phonotacticsInUse ())  {
    QList< QList< QPointer<Phoneme> > > list = phonotactics.getPeakList ();
    chooseSequenceDialog = new ChooseSequenceDialog (list, list.indexOf (syllable.getPeak ()),
                                                    false);
  }

  else
    chooseSequenceDialog = new ChooseSequenceDialog (phonotactics.getPhonemeList (),
                                                     syllable.getPeak ());

  connect (chooseSequenceDialog, SIGNAL (sequenceChosen (QList< QPointer<Phoneme> >)),
           this, SLOT (setPeak (QList< QPointer<Phoneme> >)));

  chooseSequenceDialog->exec ();
}

void EditSyllableDialog::chooseCoda ()  {
  if (chooseSequenceDialog) delete chooseSequenceDialog;

  if (phonotactics.phonotacticsInUse ())  {
    QList< QList< QPointer<Phoneme> > > list = phonotactics.getCodaList ();
    chooseSequenceDialog = new ChooseSequenceDialog (list, list.indexOf (syllable.getCoda ()),
                                                     true);
  }

  else
    chooseSequenceDialog = new ChooseSequenceDialog (phonotactics.getPhonemeList (),
                                                     syllable.getCoda ());

  connect (chooseSequenceDialog, SIGNAL (sequenceChosen (QList< QPointer<Phoneme> >)),
           this, SLOT (setCoda (QList< QPointer<Phoneme> >)));

  chooseSequenceDialog->exec ();
}

void EditSyllableDialog::setOnset (QList< QPointer<Phoneme> > list,
                                   QList< QList<bool> > check)  {
  while (onsetLabel.size () > 0)  {
    QLabel *l = onsetLabel[0];
    onsetLayout->removeWidget (l);
    onsetLabel.removeAt (0);
    delete l;
  }

  while (onsetSupraBox.size () > 0)  {
    while (onsetSupraBox[0].size () > 0)  {
      QCheckBox *c = onsetSupraBox[0][0];
      onsetLayout->removeWidget (c);
      onsetSupraBox[0].removeAt (0);
      delete c;
    }

    onsetSupraBox.removeAt (0);
  }

  onsetLayout->removeWidget (editOnsetButton);
  onsetLayout->removeWidget (onsetRightButton);

  syllable.setOnset (list);
  QList< QPointer<Phoneme> > onset = list;

  for (int x = 0; x < onset.size (); x++)  {
    onsetLabel.append (new QLabel (onset[x]->getName ()));

    onsetSupraBox.append (QList<QCheckBox*> ());

    for (int y = 0; y < supras.size (); y++)  {
      onsetSupraBox[x].append (new QCheckBox (supras[y]->getName ()));

      if (x < check.size ())
        if (y < check[x].size ())
          onsetSupraBox[x][y]->setChecked (check[x][y]);
        
      onsetSupraBox[x][y]->setEnabled (supras[y]->getPhonemeList ().contains (onset[x]));
    }

    onsetLayout->addWidget (onsetLabel[x], 1, x);
    onsetLayout->setAlignment (onsetLabel[x], Qt::AlignCenter);
    for (int y = 0; y < onsetSupraBox[x].size (); y++)
      onsetLayout->addWidget (onsetSupraBox[x][y], y + 2, x);
  }

  onsetLayout->addWidget (editOnsetButton, 0, 0, 1, onset.size ());
  onsetLayout->setAlignment (editOnsetButton, Qt::AlignCenter);
  onsetLayout->addWidget (onsetRightButton, supras.size () + 2,
                          (onset.size () > 0 ? onset.size () - 1 : 0));
  onsetRightButton->setEnabled (onset.size () > 0);
}

void EditSyllableDialog::setPeak (QList< QPointer<Phoneme> > list,
                                  QList< QList<bool> > check)  {
  while (peakLabel.size () > 0)  {
    QLabel *l = peakLabel[0];
    peakLayout->removeWidget (l);
    peakLabel.removeAt (0);
    delete l;
  }

  while (peakSupraBox.size () > 0)  {
    while (peakSupraBox[0].size () > 0)  {
      QCheckBox *c = peakSupraBox[0][0];
      peakLayout->removeWidget (c);
      peakSupraBox[0].removeAt (0);
      delete c;
    }

    peakSupraBox.removeAt (0);
  }

  peakLayout->removeWidget (editPeakButton);
  peakLayout->removeWidget (peakLeftButton);
  peakLayout->removeWidget (peakRightButton);

  syllable.setPeak (list);
  QList< QPointer<Phoneme> > peak = list;

  for (int x = 0; x < peak.size (); x++)  {
    peakLabel.append (new QLabel (peak[x]->getName ()));

    peakSupraBox.append (QList<QCheckBox*> ());

    for (int y = 0; y < supras.size (); y++)  {
      peakSupraBox[x].append (new QCheckBox (supras[y]->getName ()));

      if (x < check.size ())
        if (y < check[x].size ())
          peakSupraBox[x][y]->setChecked (check[x][y]);
      
      peakSupraBox[x][y]->setEnabled (supras[y]->getPhonemeList ().contains (peak[x]));
    }

    peakLayout->addWidget (peakLabel[x], 1, x);
    peakLayout->setAlignment (peakLabel[x], Qt::AlignCenter);
    for (int y = 0; y < peakSupraBox[x].size (); y++)
      peakLayout->addWidget (peakSupraBox[x][y], y + 2, x);
  }

  peakLayout->addWidget (editPeakButton, 0, 0, 1, peak.size ());
  peakLayout->setAlignment (editPeakButton, Qt::AlignCenter);
  peakLayout->addWidget (peakLeftButton, supras.size () + 2, 0);
  peakLayout->addWidget (peakRightButton, supras.size () + 2,
                         (peak.size () > 1 ? peak.size () - 1 : 1));
  peakLeftButton->setEnabled (peak.size () > 0);
  peakRightButton->setEnabled (peak.size () > 0);
}

void EditSyllableDialog::setCoda (QList< QPointer<Phoneme> > list,
                                  QList< QList<bool> > check)  {
  while (codaLabel.size () > 0)  {
    QLabel *l = codaLabel[0];
    codaLayout->removeWidget (l);
    codaLabel.removeAt (0);
    delete l;
  }

  while (codaSupraBox.size () > 0)  {
    while (codaSupraBox[0].size () > 0)  {
      QCheckBox *c = codaSupraBox[0][0];
      codaLayout->removeWidget (c);
      codaSupraBox[0].removeAt (0);
      delete c;
    }

    codaSupraBox.removeAt (0);
  }

  codaLayout->removeWidget (editCodaButton);
  codaLayout->removeWidget (codaLeftButton);

  syllable.setCoda (list);
  QList< QPointer<Phoneme> > coda = list;

  for (int x = 0; x < coda.size (); x++)  {
    codaLabel.append (new QLabel (coda[x]->getName ()));

    codaSupraBox.append (QList<QCheckBox*> ());

    for (int y = 0; y < supras.size (); y++)  {
      codaSupraBox[x].append (new QCheckBox (supras[y]->getName ()));

      if (x < check.size ())
        if (y < check[x].size ())
          codaSupraBox[x][y]->setChecked (check[x][y]);
      
      codaSupraBox[x][y]->setEnabled (supras[y]->getPhonemeList ().contains (coda[x]));
    }

    codaLayout->addWidget (codaLabel[x], 1, x);
    codaLayout->setAlignment (codaLabel[x], Qt::AlignCenter);
    for (int y = 0; y < codaSupraBox[x].size (); y++)
      codaLayout->addWidget (codaSupraBox[x][y], y + 2, x);
  }

  codaLayout->addWidget (editCodaButton, 0, 0, 1, coda.size ());
  codaLayout->setAlignment (editCodaButton, Qt::AlignCenter);
  codaLayout->addWidget (codaLeftButton, supras.size () + 2, 0);
  codaLeftButton->setEnabled (coda.size () > 0);
}

void EditSyllableDialog::submit ()  {
  QList< QPointer<Phoneme> > onset = syllable.getOnset ();
  QList< QList< QPointer<Suprasegmental> > > onsetSupras;

  for (int x = 0; x < onsetSupraBox.size (); x++)  {
    onsetSupras.append (QList< QPointer<Suprasegmental> > ());

    for (int y = 0; y < onsetSupraBox[x].size (); y++)
      if (onsetSupraBox[x][y]->isChecked ())
        onsetSupras[x].append (supras[y]);
  }

  syllable.setOnset (onset, onsetSupras);

  QList< QPointer<Phoneme> > peak = syllable.getPeak ();
  QList< QList< QPointer<Suprasegmental> > > peakSupras;

  for (int x = 0; x < peakSupraBox.size (); x++)  {
    peakSupras.append (QList< QPointer<Suprasegmental> > ());

    for (int y = 0; y < peakSupraBox[x].size (); y++)
      if (peakSupraBox[x][y]->isChecked ())
        peakSupras[x].append (supras[y]);
  }

  syllable.setPeak (peak, peakSupras);

  QList< QPointer<Phoneme> > coda = syllable.getCoda ();
  QList< QList< QPointer<Suprasegmental> > > codaSupras;

  for (int x = 0; x < codaSupraBox.size (); x++)  {
    codaSupras.append (QList< QPointer<Suprasegmental> > ());

    for (int y = 0; y < codaSupraBox[x].size (); y++)
      if (codaSupraBox[x][y]->isChecked ())
        codaSupras[x].append (supras[y]);
  }

  syllable.setCoda (coda, codaSupras);

  emit updateSyllable (syllableNum, syllable);
}

void EditSyllableDialog::moveOnsetRight ()  {
  QList< QPointer<Phoneme> > onset = syllable.getOnset ();
  QList< QPointer<Phoneme> > peak = syllable.getPeak ();

  if (onset.size () == 0) return;

  QList< QList<bool> > checkOnset;
  QList< QList<bool> > checkPeak;

  for (int x = 0; x < onsetSupraBox.size (); x++)  {
    checkOnset.append (QList<bool> ());
    
    for (int y = 0; y < onsetSupraBox[x].size (); y++)
      checkOnset[x].append (onsetSupraBox[x][y]->isChecked ());
  }

  for (int x = 0; x < peakSupraBox.size (); x++)  {
    checkPeak.append (QList<bool> ());

    for (int y = 0; y < peakSupraBox[x].size (); y++)
      checkPeak[x].append (peakSupraBox[x][y]->isChecked ());
  }

  peak.prepend (onset.takeLast ());
  checkPeak.prepend (checkOnset.takeLast ());

  setOnset (onset, checkOnset);
  setPeak (peak, checkPeak);
}

void EditSyllableDialog::movePeakLeft ()  {
  QList< QPointer<Phoneme> > onset = syllable.getOnset ();
  QList< QPointer<Phoneme> > peak = syllable.getPeak ();

  if (peak.size () == 0) return;

  QList< QList<bool> > checkOnset;
  QList< QList<bool> > checkPeak;

  for (int x = 0; x < onsetSupraBox.size (); x++)  {
    checkOnset.append (QList<bool> ());

    for (int y = 0; y < onsetSupraBox[x].size (); y++)
      checkOnset[x].append (onsetSupraBox[x][y]->isChecked ());
  }

  for (int x = 0; x < peakSupraBox.size (); x++)  {
    checkPeak.append (QList<bool> ());

    for (int y = 0; y < peakSupraBox[x].size (); y++)
      checkPeak[x].append (peakSupraBox[x][y]->isChecked ());
  }

  onset.append (peak.takeFirst ());
  checkOnset.append (checkPeak.takeFirst ());

  setOnset (onset, checkOnset);
  setPeak (peak, checkPeak);
}

void EditSyllableDialog::movePeakRight ()  {
  QList< QPointer<Phoneme> > coda = syllable.getCoda ();
  QList< QPointer<Phoneme> > peak = syllable.getPeak ();

  if (peak.size () == 0) return;

  QList< QList<bool> > checkCoda;
  QList< QList<bool> > checkPeak;

  for (int x = 0; x < codaSupraBox.size (); x++)  {
    checkCoda.append (QList<bool> ());

    for (int y = 0; y < codaSupraBox[x].size (); y++)
      checkCoda[x].append (codaSupraBox[x][y]->isChecked ());
  }

  for (int x = 0; x < peakSupraBox.size (); x++)  {
    checkPeak.append (QList<bool> ());

    for (int y = 0; y < peakSupraBox[x].size (); y++)
      checkPeak[x].append (peakSupraBox[x][y]->isChecked ());
  }

  coda.prepend (peak.takeLast ());
  checkCoda.prepend (checkPeak.takeLast ());

  setPeak (peak, checkPeak);
  setCoda (coda, checkCoda);
}

void EditSyllableDialog::moveCodaLeft ()  {
  QList< QPointer<Phoneme> > coda = syllable.getCoda ();
  QList< QPointer<Phoneme> > peak = syllable.getPeak ();

  if (coda.size () == 0) return;

  QList< QList<bool> > checkCoda;
  QList< QList<bool> > checkPeak;

  for (int x = 0; x < codaSupraBox.size (); x++)  {
    checkCoda.append (QList<bool> ());

    for (int y = 0; y < codaSupraBox[x].size (); y++)
      checkCoda[x].append (codaSupraBox[x][y]->isChecked ());
  }

  for (int x = 0; x < peakSupraBox.size (); x++)  {
    checkPeak.append (QList<bool> ());

    for (int y = 0; y < peakSupraBox[x].size (); y++)
      checkPeak[x].append (peakSupraBox[x][y]->isChecked ());
  }

  peak.append (coda.takeFirst ());
  checkPeak.append (checkCoda.takeFirst ());

  setPeak (peak, checkPeak);
  setCoda (coda, checkCoda);
}