#ifndef EDITSYLLABLEDIALOG_H
#define EDITSYLLABLEDIALOG_H

#include <QList>
#include <QPointer>
#include <QDialog>

#include "syllable.h"
#include "phonotactics.h"

class Phoneme;
class Suprasegmental;
class ChooseSequenceDialog;
class QGroupBox;
class QLabel;
class QPushButton;
class QCheckBox;
class QGridLayout;
class QHBoxLayout;
class QVBoxLayout;

class EditSyllableDialog : public QDialog  {
  Q_OBJECT

  public:
    EditSyllableDialog (int, Syllable, Phonotactics);

  signals:
    void updateSyllable (int, Syllable);

  private slots:
    void chooseOnset ();
    void choosePeak ();
    void chooseCoda ();

    void setOnset (QList< QPointer<Phoneme> >,
                   QList< QList<bool> > = QList< QList<bool> > ());
    void setPeak (QList< QPointer<Phoneme> >,
                  QList< QList<bool> > = QList< QList<bool> > ());
    void setCoda (QList< QPointer<Phoneme> >,
                  QList< QList<bool> > = QList< QList<bool> > ());

    void moveOnsetRight ();
    void movePeakLeft ();
    void movePeakRight ();
    void moveCodaLeft ();

    void submit ();

  private:
    Syllable syllable;
    Phonotactics phonotactics;
    int syllableNum;
    QList< QPointer<Suprasegmental> > supras;

    ChooseSequenceDialog *chooseSequenceDialog;

    QGroupBox *onsetBox;
    QGroupBox *peakBox;
    QGroupBox *codaBox;
    QList<QLabel*> onsetLabel;
    QList<QLabel*> peakLabel;
    QList<QLabel*> codaLabel;
    QList< QList<QCheckBox*> > onsetSupraBox;
    QList< QList<QCheckBox*> > peakSupraBox;
    QList< QList<QCheckBox*> > codaSupraBox;
    QPushButton *onsetRightButton;
    QPushButton *peakLeftButton;
    QPushButton *peakRightButton;
    QPushButton *codaLeftButton;
    QPushButton *editOnsetButton;
    QPushButton *editPeakButton;
    QPushButton *editCodaButton;
    QPushButton *submitButton;

    QGridLayout *onsetLayout;
    QGridLayout *peakLayout;
    QGridLayout *codaLayout;
    QHBoxLayout *syllableLayout;
    QVBoxLayout *mainLayout;
};

#endif