#ifndef PHONOLOGYPAGE_H
#define PHONOLOGYPAGE_H

#include <QWidget>
#include <QList>
#include <QPointer>

class QStringListModel;
class Phoneme;
class QPushButton;
class QListView;
class QRadioButton;
class QLineEdit;
class QTextEdit;
class QHBoxLayout;
class QVBoxLayout;
class QFont;

class PhonologyPage : public QWidget {
  Q_OBJECT
  
  public:
    PhonologyPage ();
    void setModel (QStringListModel*);
    QList< QPointer<Phoneme> > getList ();
    QList< QPointer<Phoneme> > getAlphabeticList ();
    bool useSquareBrackets ();
    void setList (QList< QPointer<Phoneme> >);
    void setAlphabeticList (QList< QPointer<Phoneme> >);

  signals:
    void phonemeListUpdated (QList< QPointer<Phoneme> >);
    void alphabetUpdated (QList< QPointer<Phoneme> >);

  public slots:
    void setPhonemeFont (QFont);
    void setOrthographyFont (QFont);
    void setNotesFont (QFont);
    void setSquareBrackets (bool);
    void clear ();

  private slots:
    void addPhoneme ();
    void displayPhoneme ();
    void updateGUI ();
    void moveUp ();
    void moveDown ();
    void deletePhoneme ();
    void editPhonemeName ();
    void editPhonemeSpelling (QString);
    void editPhonemeNotes ();
    void updateModel ();

  private:
    QList< QPointer<Phoneme> > phonemeList;
    QList< QPointer<Phoneme> > alphabetOrder;
    bool squareBrackets;

    QStringListModel *phonemeModel;
    
    QPushButton *addPhonemeButton;
    QLineEdit *addPhonemeEdit;
    QListView *phonemeListView;
    QRadioButton *parsingOrderButton;
    QRadioButton *alphabetOrderButton;
    QPushButton *moveUpButton;
    QPushButton *moveDownButton;
    QPushButton *deleteButton;
    
    QLineEdit *phonemeNameEdit;
    QPushButton *phonemeNameButton;
    QLineEdit *phonemeSpellingEdit;
    QTextEdit *notesEdit;

    QHBoxLayout *addPhonemeLayout;
    QVBoxLayout *listLayout;
    QVBoxLayout *moveUpDownLayout;
    QHBoxLayout *phonemeNameLayout;
    QHBoxLayout *phonemeSpellingLayout;
    QVBoxLayout *phonemeInfoLayout;
    QHBoxLayout *mainLayout;
};

#endif