#include <QString>
#include <QStringList>
#include <QMessageBox>
#include <QTextStream>

#include <iostream>
#include <cmath>
#include <ctime>
#include <cstdlib>
using namespace std;

#include "phonotactics.h"
#include "phoneme.h"
#include "suprasegmental.h"
#include "syllable.h"
#include "const.h"

Phonotactics::Phonotactics ()  {
  onsetRequired = false;
  usePhonotactics = true;
  ignorePhoneme = new Phoneme ("");
  ignorePhoneme->addSpelling (" ");
}

void Phonotactics::testInfiniteListAlgorithms ()  {
  if (phonemeList.size () == 0) return;
/*
  srand (time (0));
  int wordLength = rand () % 5 + 1;
  QList< QPointer<Phoneme> > word;

  for (int x = 0; x < wordLength; x++)
    word.append (phonemeList[rand () % phonemeList.size ()]);*/

  QTextStream terminal (stdout);/*
  terminal << (QString)"Test word: ";
  for (int x = 0; x < wordLength; x++)
    terminal << word[x]->getName ();

  terminal << endl;

  int index = getIndexOfSequence (word);

  terminal << (QString)"Index: " + QString::number (index) << endl;

  QList< QPointer<Phoneme> > sequence = getSequenceAtIndex (index);

  terminal << (QString)"InfiniteList[" + QString::number (index) + (QString)"]: ";
  for (int x = 0; x < sequence.size (); x++)
    terminal << sequence[x]->getName ();
  terminal << endl;*/

  for (int x = 0; x < phonemeList.size () * phonemeList.size (); x++)  {
    terminal << QString::number (x) + ": ";
    QList< QPointer<Phoneme> > sequence = getSequenceAtIndex (x);

    for (int s = 0; s < sequence.size (); s++)
      terminal << sequence[s]->getName ();

    terminal << endl;

    char c = '0';
    cin >> c;
  }
}

void Phonotactics::setOnsetRequired (bool req)  {
  onsetRequired = req;
}

void Phonotactics::setUsePhonotactics (bool use)  {
  usePhonotactics = use;
}

void Phonotactics::setPhonemeList (QList< QPointer<Phoneme> > list)  {
  phonemeList = list;

  bool isNull = false;

  for (int x = 0; x < onsetList.size (); x++)  {
    for (int y = 0; y < onsetList[x].size (); y++)
      if (onsetList[x][y].isNull ())
        isNull = true;

    if (isNull)  {
      onsetList.removeAt (x);
      x--;
      isNull = false;
    }
  }

  for (int x = 0; x < peakList.size (); x++)  {
    for (int y = 0; y < peakList[x].size (); y++)
      if (peakList[x][y].isNull ())
        isNull = true;

    if (isNull)  {
      peakList.removeAt (x);
      x--;
      isNull = false;
    }
  }

  for (int x = 0; x < codaList.size (); x++)  {
    for (int y = 0; y < codaList[x].size (); y++)
      if (codaList[x][y].isNull ())
        isNull = true;

    if (isNull)  {
      codaList.removeAt (x);
      x--;
      isNull = false;
    }
  }

  if (onsetList.size () > 0) onsetList = sortList (onsetList, true);
  if (peakList.size () > 0) peakList = sortList (peakList, true);
  if (codaList.size () > 0) codaList = sortList (codaList, false);

//  testInfiniteListAlgorithms ();
}

void Phonotactics::setSupraList (QList< QPointer<Suprasegmental> > list)  {
  phonDiacriticSupras.clear ();
  phonBeforeSupras.clear ();
  phonFollowingSupras.clear ();
  phonDoubledSupra = NULL;
  syllDiacriticSupras.clear ();
  syllBeforeSupras.clear ();
  syllFollowingSupras.clear ();
  syllDoubledSupra = NULL;

  for (int x = 0; x < list.size (); x++)  {
    if (list[x]->getDomain () == PHON_APPL)  {
      if (list[x]->getSpellingType () == DIACRITIC)
        phonDiacriticSupras.append (list[x]);
      if (list[x]->getSpellingType () == BEFORE)
        phonBeforeSupras.append (list[x]);
      if (list[x]->getSpellingType () == FOLLOWING)
        phonFollowingSupras.append (list[x]);
      if (list[x]->getSpellingType () == DOUBLED)
        phonDoubledSupra = list[x];
    }

    else  {
      if (list[x]->getSpellingType () == DIACRITIC)
        syllDiacriticSupras.append (list[x]);
      if (list[x]->getSpellingType () == BEFORE)
        syllBeforeSupras.append (list[x]);
      if (list[x]->getSpellingType () == FOLLOWING)
        syllFollowingSupras.append (list[x]);
      if (list[x]->getSpellingType () == DOUBLED)
        syllDoubledSupra = list[x];
    }
  }
}

void Phonotactics::setOnsetList (QList< QList< QPointer<Phoneme> > > list)  {
  if (usePhonotactics)
    onsetList = sortList (list, true);
}

void Phonotactics::setPeakList (QList< QList< QPointer<Phoneme> > > list)  {
  if (usePhonotactics)
    peakList = sortList (list, true);
}

void Phonotactics::setCodaList (QList< QList< QPointer<Phoneme> > > list)  {
  if (usePhonotactics)
    codaList = sortList (list, false);
}

void Phonotactics::setIgnored (QString ignored)  {
  ignorePhoneme->setSpellings ("");
  ignorePhoneme->addSpelling (" ");

  for (int x = 0; x < ignored.size (); x++)
    ignorePhoneme->addSpelling (QString (ignored[x]));
}

void Phonotactics::clear ()  {
  phonemeList.clear ();
  onsetList.clear ();
  peakList.clear ();
  codaList.clear ();
  ignorePhoneme->setSpellings ("");
  ignorePhoneme->addSpelling (" ");

  phonDiacriticSupras.clear ();
  phonBeforeSupras.clear ();
  phonFollowingSupras.clear ();
  phonDoubledSupra = NULL;
  syllDiacriticSupras.clear ();
  syllBeforeSupras.clear ();
  syllFollowingSupras.clear ();
  syllDoubledSupra = NULL;
}

bool Phonotactics::onsetIsRequired ()  {
  return onsetRequired;
}

bool Phonotactics::phonotacticsInUse ()  {
  return usePhonotactics;
}

QList< QPointer<Phoneme> > Phonotactics::getPhonemeList ()  {
  return phonemeList;
}

QList< QPointer<Suprasegmental> > Phonotactics::getSupraList ()  {
  QList< QPointer<Suprasegmental> > list;
  list << phonDiacriticSupras << phonBeforeSupras << phonFollowingSupras
       << syllDiacriticSupras << syllBeforeSupras << syllFollowingSupras;

  if (!syllDoubledSupra.isNull ())
    list.append (syllDoubledSupra);
  if (!phonDoubledSupra.isNull ())
    list.append (phonDoubledSupra);

  return list;
}

QList< QList< QPointer<Phoneme> > > Phonotactics::getOnsetList ()  {
  return onsetList;
}

QList< QList< QPointer<Phoneme> > > Phonotactics::getPeakList ()  {
  return peakList;
}

QList< QList< QPointer<Phoneme> > > Phonotactics::getCodaList ()  {
  return codaList;
}

QString Phonotactics::getIgnoreText ()  {
  QString ignore = ignorePhoneme->getSpellingsText ();
  ignore.remove (" ");
  return ignore;
}

QString Phonotactics::getSaveOutput ()  {
  QString text = "{";

  for (int x = 0; x < onsetList.size (); x++)  {
    if (onsetList[x].size () == 0) continue;

    text += "[";

    for (int y = 0; y < onsetList[x].size (); y++)
      text += onsetList[x][y]->getName ().remove (" ").remove ("]") + " ";

    text.chop (1);
    text += "] ";
  }

  text.chop (1);
  text += "}\r\n{";

  for (int x = 0; x < peakList.size (); x++)  {
    if (peakList[x].size () == 0) continue;

    text += "[";

    for (int y = 0; y < peakList[x].size (); y++)
      text += peakList[x][y]->getName ().remove (" ").remove ("]") + " ";

    text.chop (1);
    text += "] ";
  }

  text.chop (1);
  text += "}\r\n{";

  for (int x = 0; x < codaList.size (); x++)  {
    if (codaList[x].size () == 0) continue;

    text += "[";

    for (int y = 0; y < codaList[x].size (); y++)
      text += codaList[x][y]->getName ().remove (" ").remove ("]") + " ";

    text.chop (1);
    text += "] ";
  }

  text.chop (1);
  text += "}";

  return text;
}

QList<Syllable> Phonotactics::parseWord (QString word)  {
  tempSyllList.clear ();

  if (!usePhonotactics)  {
    onsetList.clear ();
    peakList.clear ();
    codaList.clear ();

    for (int x = 0; x < phonemeList.size (); x++)  {
      onsetList.append (QList< QPointer<Phoneme> > () << phonemeList[x]);
      peakList.append (QList< QPointer<Phoneme> > () << phonemeList[x]);
      codaList.append (QList< QPointer<Phoneme> > () << phonemeList[x]);
    }
  }

  QString *unparsed = new QString (word);
  int currentSyll = 0;

  while (*unparsed != "")  {
    if (tempSyllList.size () == currentSyll)
      tempSyllList.append (Syllable ());

    for (int x = 0; x < syllBeforeSupras.size (); x++)
      if (unparsed->startsWith (syllBeforeSupras[x]->getSpellingDetail ()))  {
        tempSyllList[currentSyll].addSupra (syllBeforeSupras[x]);
        unparsed->remove (0, syllBeforeSupras[x]->getSpellingDetail ().size ());
        break;
      }

    if (!parseOnset (unparsed, currentSyll, false))
      if (onsetRequired || *unparsed == "")
        return tempSyllList;

    if (!parsePeak (unparsed, currentSyll, false))
      return tempSyllList;

    for (int x = 0; x < syllFollowingSupras.size (); x++)
      if (unparsed->startsWith (syllFollowingSupras[x]->getSpellingDetail ()))  {
        tempSyllList[currentSyll].addSupra (syllFollowingSupras[x]);
        unparsed->remove (0, syllFollowingSupras[x]->getSpellingDetail ().size ());
        break;
      }

    currentSyll++;
  };

  return tempSyllList;
}

bool Phonotactics::parseOnset (QString *unparsed, int current, bool reparse)  {
//  if (usePhonotactics == false)
//    QMessageBox::information (NULL, "parseOnset", "Unparsed = " + *unparsed +
//                              "\nSyllable = " + QString::number (current) +
//                              "\nReparse = " + QString::number ((int)reparse));

  if (*unparsed == "" && !reparse) return false;

  int currentOnset = 0;

  if (reparse)  {
    currentOnset = onsetList.indexOf (tempSyllList[current].getOnset ()) + 1;
    unparsed->prepend (tempSyllList[current].getOnsetSpellings ().join (""));
    tempSyllList[current].setOnset (QList< QPointer<Phoneme> > ());
  }

  if (currentOnset < 0) currentOnset = 0;

  bool onsetFound = false;
  QStringList *retVal = new QStringList;

  for (int x = currentOnset; x < onsetList.size (); x++)
    if (trySequence (onsetList[x], *unparsed, retVal))  {
      onsetFound = true;
      tempSyllList[current].setOnset (onsetList[x], tempSupraListSequence);
      tempSyllList[current].setOnsetSpellings (*retVal);
      unparsed->remove (0, retVal->join ("").size ());
      break;
    }

  if (!onsetFound && current == 0)
    return false;

  if (!onsetFound)  {
    if (parseCoda (unparsed, current - 1))
      return parseOnset (unparsed, current, false);
    else return false;
  }

  else return true;
}

bool Phonotactics::parsePeak (QString *unparsed, int current, bool reparse)  {
//  if (usePhonotactics == false)
//    QMessageBox::information (NULL, "parsePeak", "Unparsed = " + *unparsed +
//                              "\nSyllable = " + QString::number (current) +
//                              "\nReparse = " + QString::number ((int)reparse));

  if (*unparsed == "" && !reparse)
    if (!parseOnset (unparsed, current, true))
      return false;

  int currentPeak = 0;
  if (reparse)  {
    currentPeak = peakList.indexOf (tempSyllList[current].getPeak ()) + 1;
    unparsed->prepend (tempSyllList[current].getPeakSpellings ().join (""));
    tempSyllList[current].setPeak (QList< QPointer<Phoneme> > ());
  }
  if (currentPeak < 0) currentPeak = 0;

  bool peakFound = false;
  QStringList *retVal = new QStringList;

  for (int x = currentPeak; x < peakList.size (); x++)
    if (trySequence (peakList[x], *unparsed, retVal, true))  {
      peakFound = true;
      tempSyllList[current].setPeak (peakList[x], tempSupraListSequence);
      tempSyllList[current].setPeakSpellings (*retVal);
      unparsed->remove (0, retVal->join ("").size ());
      break;
    }

  if (!peakFound)  {
    if (parseOnset (unparsed, current, true))
      return parsePeak (unparsed, current, false);
    else return false;
  }

  else return true;
}

bool Phonotactics::parseCoda (QString *unparsed, int current)  {
//  if (usePhonotactics == false)
//    QMessageBox::information (NULL, "parseCoda", "Unparsed = " + *unparsed +
//                              "\nSyllable = " + QString::number (current));

  if (codaList.size () == 0 && usePhonotactics == true) return false;

  QPointer<Suprasegmental> s = tempSyllList[current].getSupraBySpelling (FOLLOWING);

  if (!s.isNull ())  {
    unparsed->prepend (s->getSpellingDetail ());
    tempSyllList[current].removeSupra (s);
  }

  int currentCoda = 0;
  currentCoda = codaList.indexOf (tempSyllList[current].getCoda ()) + 1;
  if (currentCoda < 0) currentCoda = 0;
  unparsed->prepend (tempSyllList[current].getCodaSpellings ().join (""));
  tempSyllList[current].setCoda (QList< QPointer<Phoneme> > ());

  bool codaFound = false;
  QStringList *retVal = new QStringList;

  for (int x = currentCoda; x < codaList.size (); x++)
    if (trySequence (codaList[x], *unparsed, retVal))  {
      codaFound = true;
      tempSyllList[current].setCoda (codaList[x], tempSupraListSequence);
      tempSyllList[current].setCodaSpellings (*retVal);
      unparsed->remove (0, retVal->join ("").size ());
      break;
    }

  if (codaFound)
    for (int x = 0; x < syllFollowingSupras.size (); x++)
      if (unparsed->startsWith (syllFollowingSupras[x]->getSpellingDetail ())) {
        tempSyllList[current].addSupra (syllFollowingSupras[x]);
        unparsed->remove (0, syllFollowingSupras[x]->getSpellingDetail ().size ());
      }

  return codaFound;
}

bool Phonotactics::parseFree (QString word)  {/*
  int currentSyll = 0;

  while (word != "")  {
    if (tempSyllList.size () == currentSyll)
      tempSyllList.append (Syllable ());

    for (int x = 0; x < syllBeforeSupras.size (); x++)
      if (unparsed->startsWith (syllBeforeSupras[x]->getSpellingDetail ()))  {
        tempSyllList[currentSyll].addSupra (syllBeforeSupras[x]);
        word.remove (0, syllBeforeSupras[x]->getSpellingDetail ().size ());
        break;
      }

    if (word == "")  {
      QList< QPointer<Suprasegmental> > supraList = tempSyllList[currentSyll].getSupras ();

      if (supraList.size () > 0)  {
        word.prepend (supraList[0]->getSpellingDetail ());
        tempSyllList[currentSyll].removeSupra (supraList[0]);
      }

      else return true;
    }

    // onset
    for (int x = 0; x < phonemeList.size (); x++)  {
      QString *spelling;

      if (tryPhoneme (phonemeList[x], word, spelling, false))  {
        tempSyllList[currentSyll].setOnset (QList< QPointer<Phoneme> > (phonemeList[x]),
                                            QList< QList< QPointer<Suprasegmental> > > (tempSupraList));
        tempSyllList[currentSyll].setOnsetSpellings (QStringList (*spelling));
        word.remove (0, spelling->size ())
        break;
      }
    }

    if (word == "")  {
      // then what was just parsed as the onset needs to be parsed as the coda of the previous syllable.
      // First, we need to remove any after-syllable supras and stick

      QList
    }

    // peak
    for (int x = 0; x <= phonemeList.size (); x++)  {
      if (x == phonemeList.size ()) return false;

      QString *spelling;

      if (tryPhoneme (phonemeList[x], word, spelling, true))  {
        tempSyllList[currentSyll].setPeak (QList< QPointer<Phoneme> > (phonemeList[x]),
                                           QList< QList< QPointer<Suprasegmental> > > (tempSupraList));
        word.remove (0, spelling->size ());
        break;
      }
    }

    for (int x = 0; x < syllFollowingSupras.size (); x++)
      if (unparsed->startsWith (syllFollowingSupras[x]->getSpellingDetail ()))  {
        tempSyllList[currentSyll].addSupra (syllFollowingSupras[x]);
        unparsed->remove (0, syllFollowingSupras[x]->getSpellingDetail ().size ());
        break;
      }

    currentSyll++;
  };

  return true;*/

  return false;
}

bool Phonotactics::trySequence (QList< QPointer<Phoneme> > sequence,
                                QString word, QStringList *spelling, bool peak){
  QString *retVal = new QString ("");
  QString unparsed = word;

  spelling->clear ();
  tempSupraListSequence.clear ();

  QTextStream terminal (stdout);

  for (int x = 0; x < sequence.size (); x++)  {
    while (tryIgnored (unparsed, retVal))  {
      if (*retVal == "") break;
//      terminal << "Found ignored character: " << *retVal << endl;
      spelling->append (*retVal);
      unparsed.remove (0, retVal->size ());
    }
    
    if (tryPhoneme (sequence[x], unparsed, retVal, peak))  {
      spelling->append (*retVal);
      unparsed.remove (0, retVal->size ());
      tempSupraListSequence.append (tempSupraList);
    }

    else  {
      spelling->clear ();
      tempSupraListSequence.clear ();
      return false;
    }
  }

  if (tryIgnored (unparsed, retVal))  {
    spelling->append (*retVal);
    unparsed.remove (0, retVal->size ());
  }

  return true;
}

bool Phonotactics::tryPhoneme (QPointer<Phoneme> phoneme, QString word,
                               QString *spelling, bool peak)  {
  QStringList phonSpellings = phoneme->getSpellings ();

  tempSupraList.clear ();
  spelling->clear ();

  for (int b = 0; b < phonBeforeSupras.size (); b++)
    if (word.startsWith (phonBeforeSupras[b]->getSpellingDetail ()) &&
        phonBeforeSupras[b]->getPhonemeList ().contains (phoneme))  {
      tempSupraList.append (phonBeforeSupras[b]);
      word.remove (0, phonBeforeSupras[b]->getSpellingDetail ().size ());
      *spelling += phonBeforeSupras[b]->getSpellingDetail ();
      break;
    }

  bool phonemeFound = false;
  QString kernel = "";

  for (int x = 0; x < phonSpellings.size (); x++)  {
    QString followChar = "";
    
    if (phonSpellings[x].contains ("|") && !phonSpellings[x].startsWith ("|") &&
        !phonSpellings[x].endsWith ("|"))  {
      followChar = phonSpellings[x].split ('|')[1];
      phonSpellings[x] = phonSpellings[x].split ('|')[0];
    }
    
    if (plainChars.contains (phonSpellings[x]))  {
      for (int d = 0; d < phonDiacriticSupras.size (); d++)  {
        QString matchChar = "";

        if (phonDiacriticSupras[d]->getSpellingDetail () == "Acute Accent")
          matchChar = diacriticChars[plainChars.indexOf (phonSpellings[x])];
        if (phonDiacriticSupras[d]->getSpellingDetail () == "Grave Accent")
          matchChar = diacriticChars[plainChars.indexOf (phonSpellings[x]) + 12];
        if (phonDiacriticSupras[d]->getSpellingDetail () == "Circumflex")
          matchChar = diacriticChars[plainChars.indexOf (phonSpellings[x]) + 24];
        if (phonDiacriticSupras[d]->getSpellingDetail () == "Diaresis/Umlaut")
          matchChar = diacriticChars[plainChars.indexOf (phonSpellings[x]) + 36];
        if (phonDiacriticSupras[d]->getSpellingDetail () == "Macron")
          matchChar = diacriticChars[plainChars.indexOf (phonSpellings[x]) + 48];

        QString checkString = matchChar + followChar;

        if (word.startsWith (checkString) && checkString != "")  {
          tempSupraList.append (phonDiacriticSupras[d]);
          word.remove (0, matchChar.size ());
          kernel = matchChar;
          phonemeFound = true;
          break;
        }
      }

      if (!phonemeFound && peak)
        for (int d = 0; d < syllDiacriticSupras.size (); d++)  {
          QString matchChar = "";

          if (syllDiacriticSupras[d]->getSpellingDetail () == "Acute Accent")
            matchChar = diacriticChars[plainChars.indexOf (phonSpellings[x])];
          if (syllDiacriticSupras[d]->getSpellingDetail () == "Grave Accent")
            matchChar = diacriticChars[plainChars.indexOf (phonSpellings[x]) + 12];
          if (syllDiacriticSupras[d]->getSpellingDetail () == "Circumflex")
            matchChar = diacriticChars[plainChars.indexOf (phonSpellings[x]) + 24];
          if (syllDiacriticSupras[d]->getSpellingDetail () == "Diaresis/Umlaut")
            matchChar = diacriticChars[plainChars.indexOf (phonSpellings[x]) + 36];
          if (syllDiacriticSupras[d]->getSpellingDetail () == "Macron")
            matchChar = diacriticChars[plainChars.indexOf (phonSpellings[x]) + 48];

          QString checkChar = matchChar + followChar;

          if (word.startsWith (checkChar) && checkChar != "")  {
            tempSupraList.append (syllDiacriticSupras[d]);
            word.remove (0, matchChar.size ());
            kernel = matchChar;
            phonemeFound = true;
            break;
          }
      }
    }

    if (phonemeFound) break;

    QString checkChar = phonSpellings[x] + followChar;

    if (word.startsWith (checkChar))  {
      word.remove (0, phonSpellings[x].size ());
      kernel = phonSpellings[x];
      phonemeFound = true;
    }
  }

  if (!phonemeFound)  {
    spelling->clear ();
    tempSupraList.clear ();
    return false;
  }

  if (word.startsWith (kernel) && !phonDoubledSupra.isNull ())
    if (phonDoubledSupra->getPhonemeList ().contains (phoneme))  {
      tempSupraList.append (phonDoubledSupra);
      word.remove (0, kernel.size ());
      kernel += kernel;
    }

  if (word.startsWith (kernel) && peak && !syllDoubledSupra.isNull ())  {
    tempSupraList.append (syllDoubledSupra);
    word.remove (0, kernel.size ());
    kernel += kernel;
  }

  *spelling += kernel;

  for (int f = 0; f < phonFollowingSupras.size (); f++)
    if (word.startsWith (phonFollowingSupras[f]->getSpellingDetail ()) &&
        phonFollowingSupras[f]->getPhonemeList ().contains (phoneme))  {
      tempSupraList.append (phonFollowingSupras[f]);
      *spelling += phonFollowingSupras[f]->getSpellingDetail ();
      break;
    }

  return true;
}

bool Phonotactics::tryIgnored (QString word, QString *spelling)  {
  QStringList ignoreChars = ignorePhoneme->getSpellings ();
  
  for (int x = 0; x < ignoreChars.size (); x++)
    if (word.startsWith (ignoreChars[x]))  {
      *spelling = ignoreChars[x];
      return true;
    }

  return false;
}

QList< QPointer<Phoneme> > Phonotactics::getSequenceAtIndex (int index)  {
  int indexSoFar = 0;
  bool lengthFound = false;
  int power = 1;
  int length = 0;

  while (!lengthFound)  {
    int nextIndexSoFar = indexSoFar + pow ((double)phonemeList.size (), 
                                           (double)power);

    if (nextIndexSoFar > index)  {
      length = power;
      lengthFound = true;
    }

    else indexSoFar = nextIndexSoFar;

    power += 1;
  };

  if (length == 1) return QList< QPointer<Phoneme> > () << phonemeList[index];

  index -= indexSoFar;
  int firstPhonemeIndex = floor (index / pow ((double)phonemeList.size (), 
                                              (double)length - 1));

  int nextIndex = index % (int)(pow ((double)phonemeList.size (), 
                                     (double)length - 1));
  for (int x = 1; x <= length - 2; x++)
    nextIndex += pow ((double)phonemeList.size (), (double)x);

   return QList< QPointer<Phoneme> > () << phonemeList[firstPhonemeIndex]
                                        << getSequenceAtIndex (nextIndex);
}

int Phonotactics::getIndexOfSequence (QList< QPointer<Phoneme> > sequence)  {
  int index = 0;

  for (int x = 1; x <= sequence.size () - 1; x++)
    index += pow ((double)phonemeList.size (), (double)x);

  for (int x = 0; x < sequence.size (); x++)
    index += phonemeList.indexOf (sequence[x]) * 
             pow ((double)phonemeList.size (), 
                  (double)(sequence.size () - 1 - x));

  return index;
}

QString Phonotactics::sequenceToString (QList< QPointer<Phoneme> > sequence)  {
  QString str = "";

  for (int x = 0; x < sequence.size (); x++)
    str += sequence[x]->getName ();

  return str;
}

QList< QList< QPointer<Phoneme> > > Phonotactics::sortList
                                      (QList< QList< QPointer<Phoneme> > > list,
                                       bool longestFirst)  {
  if (phonemeList.size () == 0) return list;

  for (int x = 0; x < list.size (); x++)  {
    int first = x;

    for (int y = x + 1; y < list.size (); y++)  {
      if (list[y].size () > list[first].size () && longestFirst)
        first = y;

      else if (list[y].size () < list[first].size () && !longestFirst)
        first = y;

      else for (int p = 0; p < list[y].size () && p < list[first].size (); p++)
        if (phonemeList.indexOf (list[y][p]) >= 0 &&
            phonemeList.indexOf (list[first][p]) >= 0)
          if (phonemeList.indexOf (list[y][p]) <
              phonemeList.indexOf (list[first][p]))
            first = y;
    }

    list.swap (x, first);
  }

  return list;
}
