#ifndef PHONOLOGY_H
#define PHONOLOGY_H

#include <QList>
#include <QPointer>

#include "syllable.h"

class Phoneme;
class Suprasegmental;
class QString;
class QStringList;

class Phonotactics  {
  public:
    Phonotactics ();

    void testInfiniteListAlgorithms ();

    void setOnsetRequired (bool);
    void setUsePhonotactics (bool);
    void setPhonemeList (QList< QPointer<Phoneme> >);
    void setSupraList (QList< QPointer<Suprasegmental> >);
    void setOnsetList (QList< QList< QPointer<Phoneme> > >);
    void setPeakList (QList< QList< QPointer<Phoneme> > >);
    void setCodaList (QList< QList< QPointer<Phoneme> > >);
    void setIgnored (QString);
    void clear ();

    bool onsetIsRequired ();
    bool phonotacticsInUse ();
    QList< QPointer<Phoneme> > getPhonemeList ();
    QList< QPointer<Suprasegmental> > getSupraList ();
    QList< QList< QPointer<Phoneme> > > getOnsetList ();
    QList< QList< QPointer<Phoneme> > > getPeakList ();
    QList< QList< QPointer<Phoneme> > > getCodaList ();
    QString getIgnoreText ();
    QString getSaveOutput ();

    QList<Syllable> parseWord (QString);

  private:
    bool parseOnset (QString*, int, bool);
    bool parsePeak (QString*, int, bool);
    bool parseCoda (QString*, int);
    bool parseFree (QString);

    bool trySequence (QList< QPointer<Phoneme> >, QString, QStringList*,
                      bool = false);
    bool tryPhoneme (QPointer<Phoneme>, QString, QString*, bool);
    bool tryIgnored (QString, QString*);

    QList< QPointer<Phoneme> > getSequenceAtIndex (int);
    int getIndexOfSequence (QList< QPointer<Phoneme> >);
    QString sequenceToString (QList< QPointer<Phoneme> >);

    QList< QList< QPointer<Phoneme> > > sortList
                                    (QList< QList< QPointer<Phoneme> > >, bool);

    bool onsetRequired;
    bool usePhonotactics;

    QList<Syllable> tempSyllList;
    QList< QPointer<Suprasegmental> > tempSupraList;
    QList< QList< QPointer<Suprasegmental> > > tempSupraListSequence;

    QList< QPointer<Phoneme> > phonemeList;
    QList< QList< QPointer<Phoneme> > > onsetList;
    QList< QList< QPointer<Phoneme> > > peakList;
    QList< QList< QPointer<Phoneme> > > codaList;
    QPointer<Phoneme> ignorePhoneme;

    QList< QPointer<Suprasegmental> > phonDiacriticSupras;
    QList< QPointer<Suprasegmental> > phonBeforeSupras;
    QList< QPointer<Suprasegmental> > phonFollowingSupras;
    QPointer<Suprasegmental> phonDoubledSupra;
    QList< QPointer<Suprasegmental> > syllDiacriticSupras;
    QList< QPointer<Suprasegmental> > syllBeforeSupras;
    QList< QPointer<Suprasegmental> > syllFollowingSupras;
    QPointer<Suprasegmental> syllDoubledSupra;
};

#endif