#include <QTextEdit>
#include <QPushButton>
#include <QCheckBox>
#include <QLabel>
#include <QComboBox>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QMessageBox>
#include <QLineEdit>

#include <iostream>
using namespace std;

#include "phonotacticspage.h"
#include "phoneme.h"
#include "phonotactics.h"
#include "choosephonemesdialog.h"

PhonotacticsPage::PhonotacticsPage ()  {
  choosePhonemesDialog = NULL;

  onsetRequiredBox = new QCheckBox ("Onset Required");
  usePhonotacticsBox = new QCheckBox ("Use Phonotactics");
  QLabel *ignoreLabel = new QLabel ("Ignored Characters:");
  ignorePhonemeEdit = new QLineEdit;
  usePhonotacticsBox->setChecked (true);
  topLayout = new QHBoxLayout;
  topLayout->addWidget (onsetRequiredBox);
  topLayout->setAlignment (onsetRequiredBox, Qt::AlignLeft);
  topLayout->addWidget (ignoreLabel);
  topLayout->setAlignment (ignoreLabel, Qt::AlignRight);
  topLayout->addWidget (ignorePhonemeEdit);
  topLayout->setAlignment (ignorePhonemeEdit, Qt::AlignLeft);
  topLayout->addWidget (usePhonotacticsBox);
  topLayout->setAlignment (usePhonotacticsBox, Qt::AlignRight);

  QLabel *onsetLabel = new QLabel ("Legal Onsets:");
  onsetEdit = new QTextEdit ();
  onsetEdit->setReadOnly (true);
  addOnsetsButton = new QPushButton ("Add Onsets");
  addOnsetsButton->setMaximumSize (addOnsetsButton->sizeHint ());
  onsetLayout = new QHBoxLayout;
  onsetLayout->addWidget (onsetEdit);
  onsetLayout->addWidget (addOnsetsButton);
  onsetLayout->setAlignment (addOnsetsButton, Qt::AlignTop | Qt::AlignLeft);

  QLabel *peakLabel = new QLabel ("Legal Peaks:");
  peakEdit = new QTextEdit;
  peakEdit->setReadOnly (true);
  addPeaksButton = new QPushButton ("Add Peaks");
  peakLayout = new QHBoxLayout;
  peakLayout->addWidget (peakEdit);
  peakLayout->addWidget (addPeaksButton);
  peakLayout->setAlignment (addPeaksButton, Qt::AlignTop | Qt::AlignLeft);

  QLabel *codaLabel = new QLabel ("Legal Codas:");
  codaEdit = new QTextEdit;
  codaEdit->setReadOnly (true);
  addCodasButton = new QPushButton ("Add Codas");
  addCodasButton->setMaximumSize (addCodasButton->sizeHint ());
  codaLayout = new QHBoxLayout;
  codaLayout->addWidget (codaEdit);
  codaLayout->addWidget (addCodasButton);
  codaLayout->setAlignment (addCodasButton, Qt::AlignTop | Qt::AlignLeft);

  QLabel *clusterLabel = new QLabel ("Create Clusters:");
  addClusterListButton = new QPushButton ("Add Cluster Element");
  clearClusterListsButton = new QPushButton ("Clear Cluster Elements");
  clusterListDisplay = new QLabel ("");
  clusterListDisplay->setAlignment (Qt::AlignCenter);
  clusterListDisplay->setWordWrap (true);
  makeClustersButton = new QPushButton ("Make clusters and add them to");
  chooseListBox = new QComboBox;
  chooseListBox->addItem ("Onsets");
  chooseListBox->addItem ("Peaks");
  chooseListBox->addItem ("Codas");
  removeClustersButton = new QPushButton ("Remove clusters from");
  removeFromListBox = new QComboBox;
  removeFromListBox->addItem ("Onsets");
  removeFromListBox->addItem ("Peaks");
  removeFromListBox->addItem ("Codas");
  clusterListLayout = new QHBoxLayout;
  clusterListLayout->addWidget (addClusterListButton);
  clusterListLayout->setAlignment (addClusterListButton, Qt::AlignRight);
  clusterListLayout->addWidget (clearClusterListsButton);
  clusterListLayout->setAlignment (clearClusterListsButton, Qt::AlignLeft);
  makeClustersLayout = new QHBoxLayout;
  makeClustersLayout->addWidget (makeClustersButton);
  makeClustersLayout->setAlignment (makeClustersButton, Qt::AlignRight);
  makeClustersLayout->addWidget (chooseListBox);
  makeClustersLayout->setAlignment (chooseListBox, Qt::AlignLeft);
  removeClustersLayout = new QHBoxLayout;
  removeClustersLayout->addWidget (removeClustersButton);
  removeClustersLayout->setAlignment (removeClustersButton, Qt::AlignRight);
  removeClustersLayout->addWidget (removeFromListBox);
  removeClustersLayout->setAlignment (removeFromListBox, Qt::AlignLeft);

  savePhonotacticsButton = new QPushButton ("Save Phonotactics");
  recalcPhonologiesBox = new QCheckBox ("Update the Phonologies of all Words");

  mainLayout = new QVBoxLayout;
  mainLayout->addLayout (topLayout);
  mainLayout->addWidget (onsetLabel);
  mainLayout->setAlignment (onsetLabel, Qt::AlignLeft);
  mainLayout->addLayout (onsetLayout);
  mainLayout->addWidget (peakLabel);
  mainLayout->setAlignment (peakLabel, Qt::AlignLeft);
  mainLayout->addLayout (peakLayout);
  mainLayout->addWidget (codaLabel);
  mainLayout->setAlignment (codaLabel, Qt::AlignLeft);
  mainLayout->addLayout (codaLayout);
  mainLayout->insertSpacing (-1, 20);
  mainLayout->addWidget (clusterLabel);
  mainLayout->setAlignment (clusterLabel, Qt::AlignCenter);
  mainLayout->addLayout (clusterListLayout);
  mainLayout->addWidget (clusterListDisplay);
  mainLayout->addLayout (makeClustersLayout);
  mainLayout->addLayout (removeClustersLayout);
  mainLayout->insertSpacing (-1, 30);
  mainLayout->addWidget (savePhonotacticsButton);
  mainLayout->setAlignment (savePhonotacticsButton, Qt::AlignCenter);
  mainLayout->addWidget (recalcPhonologiesBox);
  mainLayout->setAlignment (recalcPhonologiesBox, Qt::AlignCenter);

  setLayout (mainLayout);

  connect (addOnsetsButton, SIGNAL (clicked ()), this, SLOT (chooseOnsets ()));
  connect (addPeaksButton, SIGNAL (clicked ()), this, SLOT (choosePeaks ()));
  connect (addCodasButton, SIGNAL (clicked ()), this, SLOT (chooseCodas ()));
  connect (addClusterListButton, SIGNAL (clicked ()), this,
           SLOT (chooseClusterList ()));
  connect (clearClusterListsButton, SIGNAL (clicked ()), this,
           SLOT (clearClusterLists ()));
  connect (makeClustersButton, SIGNAL (clicked ()), this,
           SLOT (makeClusters ()));
  connect (removeClustersButton, SIGNAL (clicked ()), this,
           SLOT (removeClusters ()));
  connect (savePhonotacticsButton, SIGNAL (clicked ()), this,
           SLOT (savePhonotactics ()));

  connect (usePhonotacticsBox, SIGNAL (toggled (bool)), onsetRequiredBox, SLOT (setEnabled (bool)));
  connect (usePhonotacticsBox, SIGNAL (toggled (bool)), addOnsetsButton, SLOT (setEnabled (bool)));
  connect (usePhonotacticsBox, SIGNAL (toggled (bool)), addPeaksButton, SLOT (setEnabled (bool)));
  connect (usePhonotacticsBox, SIGNAL (toggled (bool)), addCodasButton, SLOT (setEnabled (bool)));
  connect (usePhonotacticsBox, SIGNAL (toggled (bool)), addClusterListButton, SLOT (setEnabled (bool)));
  connect (usePhonotacticsBox, SIGNAL (toggled (bool)), clearClusterListsButton, SLOT (setEnabled (bool)));
  connect (usePhonotacticsBox, SIGNAL (toggled (bool)), makeClustersButton, SLOT (setEnabled (bool)));
  connect (usePhonotacticsBox, SIGNAL (toggled (bool)), chooseListBox, SLOT (setEnabled (bool)));
  connect (usePhonotacticsBox, SIGNAL (toggled (bool)), removeClustersButton, SLOT (setEnabled (bool)));
  connect (usePhonotacticsBox, SIGNAL (toggled (bool)), removeFromListBox, SLOT (setEnabled (bool)));
  connect (usePhonotacticsBox, SIGNAL (toggled (bool)), this, SLOT (clear ()));
}

void PhonotacticsPage::setPhonotactics (Phonotactics tax)  {
  ignorePhonemeEdit->setText (tax.getIgnoreText ());
  phonemeList = tax.getPhonemeList ();
  onsetList = tax.getOnsetList ();
  peakList = tax.getPeakList ();
  codaList = tax.getCodaList ();
  clusterLists.clear ();
  updateDisplay ();
}

void PhonotacticsPage::setPhonemeList (QList< QPointer<Phoneme> > list)  {
  phonemeList = list;

  clusterLists.clear ();

  bool isNull = false;

  for (int x = 0; x < onsetList.size (); x++)  {
    for (int y = 0; y < onsetList[x].size (); y++)
      if (onsetList[x][y].isNull ())
        isNull = true;

    if (isNull)  {
      onsetList.removeAt (x);
      isNull = false;
      x--;
    }
  }

  for (int x = 0; x < peakList.size (); x++)  {
    for (int y = 0; y < peakList[x].size (); y++)
      if (peakList[x][y].isNull ())
        isNull = true;

    if (isNull)  {
      peakList.removeAt (x);
      isNull = false;
      x--;
    }
  }

  for (int x = 0; x < codaList.size (); x++)  {
    for (int y = 0; y < codaList[x].size (); y++)
      if (codaList[x][y].isNull ())
        isNull = true;

    if (isNull)  {
      codaList.removeAt (x);
      isNull = false;
      x--;
    }
  }

  updateDisplay ();
}

void PhonotacticsPage::setPhonemeFont (QFont font)  {
  onsetEdit->setFont (font);
  peakEdit->setFont (font);
  codaEdit->setFont (font);
  clusterListDisplay->setFont (font);
}

void PhonotacticsPage::setOrthographyFont (QFont font)  {
  ignorePhonemeEdit->setFont (font);
}

void PhonotacticsPage::clear ()  {
  ignorePhonemeEdit->clear ();
  onsetList.clear ();
  peakList.clear ();
  codaList.clear ();
  clusterLists.clear ();
  updateDisplay ();
}

void PhonotacticsPage::chooseOnsets ()  {
  if (phonemeList.isEmpty ()) return;

  if (choosePhonemesDialog)
    delete choosePhonemesDialog;

  QList< QPointer<Phoneme> > singleOnsets;
  for (int x = 0; x < onsetList.size (); x++)
    if (onsetList[x].size () == 1)
      singleOnsets.append (onsetList[x][0]);

  choosePhonemesDialog = new ChoosePhonemesDialog (phonemeList, singleOnsets);
  choosePhonemesDialog->setPhonemeFont (onsetEdit->font ());

  connect (choosePhonemesDialog,
           SIGNAL (chosenPhonemes (QList< QPointer<Phoneme> >)), this,
           SLOT (addOnsets (QList< QPointer<Phoneme> >)));

  choosePhonemesDialog->exec ();
}

void PhonotacticsPage::choosePeaks ()  {
  if (phonemeList.isEmpty ()) return;

  if (choosePhonemesDialog)
    delete choosePhonemesDialog;

  QList< QPointer<Phoneme> > singlePeaks;
  for (int x = 0; x < peakList.size (); x++)
    if (peakList[x].size () == 1)
      singlePeaks.append (peakList[x][0]);

  choosePhonemesDialog = new ChoosePhonemesDialog (phonemeList, singlePeaks);
  choosePhonemesDialog->setFont (onsetEdit->font ());

  connect (choosePhonemesDialog,
           SIGNAL (chosenPhonemes (QList< QPointer<Phoneme> >)), this,
           SLOT (addPeaks (QList< QPointer<Phoneme> >)));

  choosePhonemesDialog->exec ();
}

void PhonotacticsPage::chooseCodas ()  {
  if (phonemeList.isEmpty ()) return;

  if (choosePhonemesDialog)
    delete choosePhonemesDialog;

  QList< QPointer<Phoneme> > singleCodas;
  for (int x = 0; x < codaList.size (); x++)
    if (codaList[x].size () == 1)
      singleCodas.append (codaList[x][0]);

  choosePhonemesDialog = new ChoosePhonemesDialog (phonemeList, singleCodas);
  choosePhonemesDialog->setFont (onsetEdit->font ());

  connect (choosePhonemesDialog,
           SIGNAL (chosenPhonemes (QList< QPointer<Phoneme> >)), this,
           SLOT (addCodas (QList< QPointer<Phoneme> >)));

  choosePhonemesDialog->exec ();
}

void PhonotacticsPage::chooseClusterList ()  {
  if (phonemeList.isEmpty ()) return;

  if (choosePhonemesDialog)
    delete choosePhonemesDialog;

  choosePhonemesDialog =
          new ChoosePhonemesDialog (phonemeList, QList< QPointer<Phoneme> > ());
  choosePhonemesDialog->setFont (onsetEdit->font ());

  connect (choosePhonemesDialog,
           SIGNAL (chosenPhonemes (QList< QPointer<Phoneme> >)), this,
           SLOT (addClusterList (QList< QPointer<Phoneme> >)));

  choosePhonemesDialog->exec ();
}

void PhonotacticsPage::addOnsets (QList< QPointer<Phoneme> > list)  {
  for (int x = 0; x < list.size (); x++)
    if (!onsetList.contains (QList< QPointer<Phoneme> > () << list[x]))
      onsetList.append (QList< QPointer<Phoneme> > () << list[x]);

  updateDisplay ();
}

void PhonotacticsPage::addPeaks (QList< QPointer<Phoneme> > list)  {
  for (int x = 0; x < list.size (); x++)
    if (!peakList.contains (QList< QPointer<Phoneme> > () << list[x]))
      peakList.append (QList< QPointer<Phoneme> > () << list[x]);

  updateDisplay ();
}

void PhonotacticsPage::addCodas (QList< QPointer<Phoneme> > list)  {
  for (int x = 0; x < list.size (); x++)
    if (!codaList.contains (QList< QPointer<Phoneme> > () << list[x]))
      codaList.append (QList< QPointer<Phoneme> > () << list[x]);

  updateDisplay ();
}

void PhonotacticsPage::addClusterList (QList< QPointer<Phoneme> > list)  {
  clusterLists.append (list);
  updateDisplay ();
}

void PhonotacticsPage::clearClusterLists ()  {
  clusterLists.clear ();
  updateDisplay ();
}

void PhonotacticsPage::makeClusters ()  {
  if (clusterLists.isEmpty ()) return;

  tempClusterList.clear ();

  int numClusters = 1;
  for (int x = 0; x < clusterLists.size (); x++)
    numClusters *= clusterLists[x].size ();

  if (numClusters > 1000)  {
    QMessageBox::warning (this, "Too Many Clusters",
                          (QString)"You're generating more than 1000 clusters.  Large" +
                          (QString)" numbers of clusters could take a huge amount of " +
                          (QString)"time to generate.  Consider simply turning off " +
                          (QString)"phonotactics instead.");
    return;
  }

  for (int x = 0; x < clusterLists[0].size (); x++)
    makeClusterPermutation (QList< QPointer<Phoneme> > (), clusterLists[0][x]);

  if (chooseListBox->currentText () == "Onsets")
    for (int x = 0; x < tempClusterList.size (); x++)
      if (!onsetList.contains (tempClusterList[x]))
        onsetList.append (tempClusterList[x]);

  if (chooseListBox->currentText () == "Peaks")
    for (int x = 0; x < tempClusterList.size (); x++)
      if (!peakList.contains (tempClusterList[x]))
        peakList.append (tempClusterList[x]);

  if (chooseListBox->currentText () == "Codas")
    for (int x = 0; x < tempClusterList.size (); x++)
      if (!codaList.contains (tempClusterList[x]))
        codaList.append (tempClusterList[x]);

  tempClusterList.clear ();
  updateDisplay ();
}

void PhonotacticsPage::removeClusters ()  {
  if (clusterLists.isEmpty ()) return;

  tempClusterList.clear ();

  int numClusters = 1;
  for (int x = 0; x < clusterLists.size (); x++)
    numClusters *= clusterLists[x].size ();

  if (numClusters > 1000)  {
    QMessageBox::warning (this, "Too Many Clusters",
                          (QString)"You're generating more than 1000 clusters.  Large" +
                          (QString)" numbers of clusters could take a huge amount of " +
                          (QString)"time to generate.  Consider simply turning off " +
                          (QString)"phonotactics instead.");
    return;
  }

  for (int x = 0; x < clusterLists[0].size (); x++)
    makeClusterPermutation (QList< QPointer<Phoneme> > (), clusterLists[0][x]);

  if (removeFromListBox->currentText () == "Onsets")
    for (int x = 0; x < tempClusterList.size (); x++)
      onsetList.removeAll (tempClusterList[x]);

  if (removeFromListBox->currentText () == "Peaks")
    for (int x = 0; x < tempClusterList.size (); x++)
      peakList.removeAll (tempClusterList[x]);

  if (removeFromListBox->currentText () == "Codas")
    for (int x = 0; x < tempClusterList.size (); x++)
      codaList.removeAll (tempClusterList[x]);

  tempClusterList.clear ();
  updateDisplay ();
}

void PhonotacticsPage::savePhonotactics ()  {
  Phonotactics phonotactics;
  phonotactics.setOnsetRequired (onsetRequiredBox->isChecked ());
  phonotactics.setUsePhonotactics (usePhonotacticsBox->isChecked ());
  phonotactics.setPhonemeList (phonemeList);
  phonotactics.setOnsetList (onsetList);
  phonotactics.setPeakList (peakList);
  phonotactics.setCodaList (codaList);
  phonotactics.setIgnored (ignorePhonemeEdit->text ());

  emit phonotacticsUpdated (phonotactics, recalcPhonologiesBox->isChecked ());
}

void PhonotacticsPage::makeClusterPermutation (QList< QPointer<Phoneme> > cluster,
                                               QPointer<Phoneme> next)  {
  if (!next.isNull ()) cluster.append (next);

  int currentList = cluster.size ();

  if (currentList == clusterLists.size ())  {
    tempClusterList.append (cluster);
    return;
  }

  for (int x = 0; x < clusterLists[currentList].size (); x++)
    makeClusterPermutation (cluster, clusterLists[currentList][x]);
}

void PhonotacticsPage::updateDisplay ()  {
  QString text = "";

  for (int x = 0; x < onsetList.size (); x++)  {
    for (int y = 0; y < onsetList[x].size (); y++)
      text += onsetList[x][y]->getName ();
    text += " ";
  }

  text.chop (1);
  onsetEdit->setPlainText (text);
  text = "";

  for (int x = 0; x < peakList.size (); x++)  {
    for (int y = 0; y < peakList[x].size (); y++)
      text += peakList[x][y]->getName ();
    text += " ";
  }

  text.chop (1);
  peakEdit->setPlainText (text);
  text = "";

  for (int x = 0; x < codaList.size (); x++)  {
    for (int y = 0; y < codaList[x].size (); y++)
      text += codaList[x][y]->getName ();
    text += " ";
  }

  text.chop (1);
  codaEdit->setPlainText (text);
  text = "";

  for (int x = 0; x < clusterLists.size (); x++)  {
    text += "{";

    for (int y = 0; y < clusterLists[x].size (); y++)
      text += clusterLists[x][y]->getName () + " ";

    text.chop (1);
    text += "} + ";
  }

  text.chop (3);
  clusterListDisplay->setText (text);
}