#ifndef PHONOTACTICSPAGE_H
#define PHONOTACTICSPAGE_H

#include <QWidget>
#include <QList>
#include <QPointer>

#include "phonotactics.h"

class Phoneme;
class ChoosePhonemesDialog;
class QTextEdit;
class QPushButton;
class QCheckBox;
class QLabel;
class QComboBox;
class QLineEdit;
class QHBoxLayout;
class QVBoxLayout;

class PhonotacticsPage : public QWidget  {
  Q_OBJECT

  public:
    PhonotacticsPage ();
    void setPhonotactics (Phonotactics);

  signals:
    void phonotacticsUpdated (Phonotactics, bool);

  public slots:
    void setPhonemeList (QList< QPointer<Phoneme> >);
    void setPhonemeFont (QFont);
    void setOrthographyFont (QFont);
    void clear ();

  private slots:
    void chooseOnsets ();
    void choosePeaks ();
    void chooseCodas ();
    void chooseClusterList ();

    void addOnsets (QList< QPointer<Phoneme> >);
    void addPeaks (QList< QPointer<Phoneme> >);
    void addCodas (QList< QPointer<Phoneme> >);

    void addClusterList (QList< QPointer<Phoneme> >);
    void clearClusterLists ();
    void makeClusters ();
    void removeClusters ();

    void savePhonotactics ();

  private:
    void makeClusterPermutation (QList< QPointer<Phoneme> >, QPointer<Phoneme>);

    void updateDisplay ();

    QList< QPointer<Phoneme> > phonemeList;
    QList< QList< QPointer<Phoneme> > > onsetList;
    QList< QList< QPointer<Phoneme> > > peakList;
    QList< QList< QPointer<Phoneme> > > codaList;
    QList< QList< QPointer<Phoneme> > > clusterLists;
    QList< QList< QPointer<Phoneme> > > tempClusterList;

    ChoosePhonemesDialog *choosePhonemesDialog;

    QCheckBox *onsetRequiredBox;
    QCheckBox *usePhonotacticsBox;
    QLineEdit *ignorePhonemeEdit;

    QTextEdit *onsetEdit;
    QPushButton *addOnsetsButton;
    QTextEdit *peakEdit;
    QPushButton *addPeaksButton;
    QTextEdit *codaEdit;
    QPushButton *addCodasButton;

    QPushButton *addClusterListButton;
    QPushButton *clearClusterListsButton;
    QLabel *clusterListDisplay;
    QPushButton *makeClustersButton;
    QComboBox *chooseListBox;
    QPushButton *removeClustersButton;
    QComboBox *removeFromListBox;

    QPushButton *savePhonotacticsButton;
    QCheckBox *recalcPhonologiesBox;

    QHBoxLayout *topLayout;
    QHBoxLayout *onsetLayout;
    QHBoxLayout *peakLayout;
    QHBoxLayout *codaLayout;
    QHBoxLayout *clusterListLayout;
    QHBoxLayout *makeClustersLayout;
    QHBoxLayout *removeClustersLayout;
    QVBoxLayout *mainLayout;
};

#endif