#include <QLineEdit>
#include <QPushButton>
#include <QComboBox>
#include <QTextEdit>
#include <QString>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QTextStream>

#include <iostream>
using namespace std;

#include "searchpage.h"
#include "word.h"

SearchPage::SearchPage ()  {
  orthographyFont = this->font ();
  notesFont = this->font ();
  
  searchButton = new QPushButton ("Search for");
  searchButton->setMaximumSize (searchButton->sizeHint ());
  searchBox = new QLineEdit;
  typeBox = new QComboBox;
  typeBox->addItem ("in Words");
  typeBox->addItem ("in Definitions");
  searchLayout = new QHBoxLayout;
  searchLayout->addWidget (searchButton);
  searchLayout->addWidget (searchBox);
  searchLayout->addWidget (typeBox);

  resultsDisplay = new QTextEdit;
  resultsDisplay->setReadOnly (true);

  mainLayout = new QVBoxLayout;
  mainLayout->addLayout (searchLayout);
  mainLayout->addWidget (resultsDisplay);

  setLayout (mainLayout);

  connect (searchButton, SIGNAL (clicked ()), this, SLOT (search ()));
}

void SearchPage::setWordList (QList< QPointer<Word> > list)  {
  wordList = list;
}

void SearchPage::setOrthographyFont (QFont font)  {
  orthographyFont = font;
}

void SearchPage::setNotesFont (QFont font)  {
  notesFont = font;
}

void SearchPage::clear ()  {
  wordList.clear ();
}

void SearchPage::search ()  {
  QString results = "";

  for (int x = 0; x < wordList.size (); x++)  {
    if ((typeBox->currentIndex () == 0 && wordList[x]->getName ().contains (searchBox->text ())) ||
        (typeBox->currentIndex () == 1 && wordList[x]->getDefinition ().contains (searchBox->text ())))  {
      results += "<b><font face=" + orthographyFont.family () + ">" + wordList[x]->getName () +
                 "</font></b> (<i><font face=" + notesFont.family () + ">"+ wordList[x]->getType ();
    
      if (wordList[x]->getSubType () != "")
        results += ", " + wordList[x]->getSubType () + "</i>) " + wordList[x]->getDefinition () +"</font><br>";
      else results += "</i>) " + wordList[x]->getDefinition () + "</font><br>";
    }
  }

  resultsDisplay->setHtml (results);
}