#include <QLineEdit>
#include <QLabel>
#include <QComboBox>
#include <QPushButton>
#include <QString>
#include <QLayoutItem>
#include <QHBoxLayout>
#include <QVBoxLayout>

#include <iostream>
using namespace std;

#include "const.h"
#include "setparadigmsdialog.h"

SetParadigmsDialog::SetParadigmsDialog (QStringList para, QList<int> use)  {
  numParadigms = para.size ();
  
  for (int x = 0; x < numParadigms; x++)  {
    QLabel *slash = new QLabel ("\\");
    
    paradigmEdit.append (new QLineEdit);
    paradigmEdit[x]->setText (para[x]);
    
    useBox.append (new QComboBox);
    useBox[x]->addItems (wordFields);
    if (use.size () > x) useBox[x]->setCurrentIndex (use[x]);

    paradigmLayout.append (new QHBoxLayout);
    paradigmLayout[x]->addWidget (slash);
    paradigmLayout[x]->addWidget (paradigmEdit[x]);
    paradigmLayout[x]->addWidget (useBox[x]);

    paradigmWidget.append (new QWidget);
    paradigmWidget[x]->setLayout (paradigmLayout[x]);
  }

  moreButton = new QPushButton ("More");
  moreButton->setMaximumSize (moreButton->sizeHint ());
  lessButton = new QPushButton ("Fewer");
  lessButton->setMaximumSize (lessButton->sizeHint ());
  lessButton->setEnabled (numParadigms > 1);
  moreLessLayout = new QHBoxLayout;
  moreLessLayout->addWidget (moreButton);
  moreLessLayout->setAlignment (moreButton, Qt::AlignRight);
  moreLessLayout->addWidget (lessButton);
  moreLessLayout->setAlignment (lessButton, Qt::AlignLeft);

  submitButton = new QPushButton ("Submit");
  submitButton->setMaximumSize (submitButton->sizeHint ());

  mainLayout = new QVBoxLayout;
  for (int x = 0; x < numParadigms; x++)
    mainLayout->addWidget (paradigmWidget[x]);
  mainLayout->addLayout (moreLessLayout);
  mainLayout->addWidget (submitButton);
  mainLayout->setAlignment (submitButton, Qt::AlignCenter);
  mainLayout->setSizeConstraint (QLayout::SetFixedSize);

  setLayout (mainLayout);

  connect (moreButton, SIGNAL (clicked ()), this, SLOT (more ()));
  connect (lessButton, SIGNAL (clicked ()), this, SLOT (less ()));
  connect (submitButton, SIGNAL (clicked ()), this, SLOT (submit ()));
  connect (submitButton, SIGNAL (clicked ()), this, SLOT (hide ()));
}

void SetParadigmsDialog::more ()  {
  numParadigms++;
  updateDisplay ();
}

void SetParadigmsDialog::less ()  {
  if (numParadigms <= 1) return;
  
  numParadigms--;
  updateDisplay ();
}

void SetParadigmsDialog::submit ()  {
  QStringList paraList;
  QList<int> useList;

  for (int x = 0; x < numParadigms; x++)  {
    if (paradigmEdit.size () <= x || useBox.size () <= x)  {
      cout << "SetParadigmsDialog::submit: lists are out of sync." << endl;
      break;
    }
    
    paraList.append (paradigmEdit[x]->text ());
    useList.append (useBox[x]->currentIndex ());
  }

  emit submit (paraList, useList);
}

void SetParadigmsDialog::updateDisplay ()  {
  while (paradigmWidget.size () < numParadigms)  {
    paradigmWidget.append (new QWidget);
    paradigmLayout.append (new QHBoxLayout);
    paradigmEdit.append (new QLineEdit);
    useBox.append (new QComboBox);
    useBox.last ()->addItems (wordFields);
    QLabel *slash = new QLabel ("\\");

    paradigmLayout.last ()->addWidget (slash);
    paradigmLayout.last ()->addWidget (paradigmEdit.last ());
    paradigmLayout.last ()->addWidget (useBox.last ());
    paradigmWidget.last ()->setLayout (paradigmLayout.last ());
  }

  QLayoutItem *child = NULL;
  while ((child = mainLayout->takeAt (0)) != NULL)  {
    if (child->widget () != NULL)
      child->widget ()->setParent (NULL);
  }

  for (int x = 0; x < numParadigms; x++)
    mainLayout->addWidget (paradigmWidget[x]);
  mainLayout->addLayout (moreLessLayout);
  mainLayout->addWidget (submitButton);
  mainLayout->setAlignment (submitButton, Qt::AlignCenter);

  lessButton->setEnabled (numParadigms > 1);
}