#include <QString>

#include "suprasegmental.h"
#include "phoneme.h"
#include "const.h"

Suprasegmental::Suprasegmental ()  {
  name = "";
  domain = PHON_APPL;
  spellingType = 0;
  spellingDetail = "";
  representationType = 0;
  representationDetail = "";
  description = "";
}

QString Suprasegmental::getName ()  {
  return name;
}

QString Suprasegmental::getPhonemesText ()  {
  QString text = "";

  for (int x = 0; x < applicablePhonemes.size (); x++)
    text += applicablePhonemes[x]->getName () + " ";

  text.chop (1);
  return text;
}

QList< QPointer<Phoneme> > Suprasegmental::getPhonemeList ()  {
  return applicablePhonemes;
}

int Suprasegmental::getDomain ()  {
  return domain;
}

int Suprasegmental::getSpellingType ()  {
  return spellingType;
}

QString Suprasegmental::getSpellingDetail ()  {
  return spellingDetail;
}

int Suprasegmental::getRepresentationType ()  {
  return representationType;
}

QString Suprasegmental::getRepresentationDetail ()  {
  return representationDetail;
}

QString Suprasegmental::getDescription ()  {
  return description;
}

QString Suprasegmental::getSaveOutput ()  {
  QString text = "";

  text += "\"" + name.replace ("\"", "\\\"") + "\",";
  text += getPhonemesText () + ",";
  text += QString::number (domain) + "," + QString::number (spellingType) + ",";
  text += "\"" + spellingDetail.replace ("\"", "\\\"")  + "\",";
  text += QString::number (representationType) + ",\"";
  text += representationDetail.replace ("\"", "\\\"") + "\"";
  text += description.replace (QRegExp ("\r?\n"), ";;");

  return text;
}

void Suprasegmental::setName (QString n)  {
  name = n;
}

void Suprasegmental::setPhonemeList (QList< QPointer<Phoneme> > list)  {
  applicablePhonemes = list;
}

void Suprasegmental::setDomain (int d)  {
  domain = d;
}

void Suprasegmental::setSpellingRules (int t, QString d)  {
  spellingType = t;
  spellingDetail = d;
}

void Suprasegmental::setRepresentationRules (int t, QString d)  {
  representationType = t;
  representationDetail = d;
}

void Suprasegmental::setDescription (QString d)  {
  description = d;
}

void Suprasegmental::removeDuds ()  {
  for (int x = 0; x < applicablePhonemes.size (); x++)
    if (applicablePhonemes[x].isNull ())  {
      applicablePhonemes.removeAt (x);
      x--;
    }
}