#include <QStringListModel>
#include <QListView>
#include <QPushButton>
#include <QLineEdit>
#include <QLabel>
#include <QGroupBox>
#include <QRadioButton>
#include <QComboBox>
#include <QTextEdit>
#include <QTextStream>
#include <QVBoxLayout>
#include <QHBoxLayout>

#include <iostream>
using namespace std;

#include "suprasegmentalspage.h"
#include "choosephonemesdialog.h"
#include "suprasegmental.h"
#include "phoneme.h"
#include "const.h"

SuprasegmentalsPage::SuprasegmentalsPage ()  {
  suprasegmentalModel = new QStringListModel;

  choosePhonemesDialog = NULL;

  addSuprasegmentalButton = new QPushButton ("Add Suprasegmental:");
  addSuprasegmentalButton->setMaximumSize
                                      (addSuprasegmentalButton->sizeHint ());
  addSuprasegmentalEdit = new QLineEdit;
  addSuprasegmentalEdit->setMinimumWidth (100);
  addSuprasegmentalLayout = new QHBoxLayout;
  addSuprasegmentalLayout->addWidget (addSuprasegmentalButton);
  addSuprasegmentalLayout->addWidget (addSuprasegmentalEdit);

  suprasegmentalListView = new QListView;
  suprasegmentalListView->setModel (suprasegmentalModel);
  suprasegmentalListView->setEditTriggers (QAbstractItemView::NoEditTriggers);
  suprasegmentalListView->setSelectionMode (QAbstractItemView::ExtendedSelection);
  suprasegmentalListLayout = new QVBoxLayout;
  suprasegmentalListLayout->addLayout (addSuprasegmentalLayout);
  suprasegmentalListLayout->addWidget (suprasegmentalListView);

  moveUpButton = new QPushButton ("Move Up");
  moveUpButton->setMaximumSize (moveUpButton->sizeHint ());
  moveUpButton->setEnabled (false);
  moveDownButton = new QPushButton ("Move Down");
  moveDownButton->setMaximumSize (moveDownButton->sizeHint ());
  moveDownButton->setEnabled (false);
  deleteButton = new QPushButton ("Delete");
  deleteButton->setMaximumSize (deleteButton->sizeHint ());
  deleteButton->setEnabled (false);
  moveUpDownLayout = new QVBoxLayout;
  moveUpDownLayout->addStretch (2);
  moveUpDownLayout->addWidget (moveUpButton);
  moveUpDownLayout->setAlignment (moveUpButton, Qt::AlignCenter);
  moveUpDownLayout->addStretch (1);
  moveUpDownLayout->addWidget (moveDownButton);
  moveUpDownLayout->addStretch (1);
  moveUpDownLayout->addWidget (deleteButton);
  moveUpDownLayout->setAlignment (deleteButton, Qt::AlignCenter);

  QLabel *nameLabel = new QLabel ("Name:");
  nameEdit = new QLineEdit;
  editNameButton = new QPushButton ("Change");
  editNameButton->setMaximumSize (editNameButton->sizeHint ());
  editNameButton->setEnabled (false);
  nameLayout = new QHBoxLayout;
  nameLayout->addWidget (nameLabel);
  nameLayout->addWidget (nameEdit);
  nameLayout->addWidget (editNameButton);

  phonApplyButton = new QRadioButton ("Applies to Phonemes");
  phonApplyButton->setEnabled (false);
  syllApplyButton = new QRadioButton ("Applies to Syllables");
  syllApplyButton->setEnabled (false);
  domainLayout = new QHBoxLayout;
  domainLayout->addWidget (phonApplyButton);
  domainLayout->addWidget (syllApplyButton);

  QLabel *phonemesLabel = new QLabel ("Applicable Phonemes:");
  applicablePhonemesLabel = new QLabel;
  editPhonemesButton = new QPushButton ("Edit Phonemes");
  editPhonemesButton->setMaximumSize (editPhonemesButton->sizeHint ());
  editPhonemesButton->setEnabled (false);
  phonemesLayout = new QHBoxLayout;
  phonemesLayout->addWidget (phonemesLabel);
  phonemesLayout->addWidget (applicablePhonemesLabel);
  phonemesLayout->addWidget (editPhonemesButton);

  spellingDiacriticButton = new QRadioButton
                                        ("Add Diacritic (a e i o u y only):");
  spellingDiacriticButton->setMinimumSize(spellingDiacriticButton->sizeHint ());
  spellingDiacriticButton->setChecked (true);
  spellingDiacriticBox = new QComboBox;
  spellingDiacriticBox->addItem ("Acute Accent");
  spellingDiacriticBox->addItem ("Grave Accent");
  spellingDiacriticBox->addItem ("Circumflex");
  spellingDiacriticBox->addItem ("Diaresis/Umlaut");
  spellingDiacriticBox->addItem ("Macron");
  spellingDiacriticLayout = new QHBoxLayout;
  spellingDiacriticLayout->addWidget (spellingDiacriticButton);
  spellingDiacriticLayout->addWidget (spellingDiacriticBox);

  spellingCharacterBeforeButton = new QRadioButton ("Character before:");
  spellingCharacterBeforeEdit = new QLineEdit;
  spellingCharacterBeforeEdit->setEnabled (false);
  spellingCharacterBeforeLayout = new QHBoxLayout;
  spellingCharacterBeforeLayout->addWidget (spellingCharacterBeforeButton);
  spellingCharacterBeforeLayout->addWidget (spellingCharacterBeforeEdit);

  spellingCharacterFollowingButton = new QRadioButton ("Character following:");
  spellingCharacterFollowingEdit = new QLineEdit;
  spellingCharacterFollowingEdit->setEnabled (false);
  spellingCharacterFollowingLayout = new QHBoxLayout;
  spellingCharacterFollowingLayout->addWidget
                                            (spellingCharacterFollowingButton);
  spellingCharacterFollowingLayout->addWidget (spellingCharacterFollowingEdit);

  spellingCharacterDoubledButton = new QRadioButton ("Character doubled");
  updateSpellingButton = new QPushButton ("Update");
  updateSpellingButton->setMaximumSize (updateSpellingButton->sizeHint ());
  updateSpellingButton->setEnabled (false);
  spellingGroupBox = new QGroupBox ("Spelling");
  spellingLayout = new QVBoxLayout;
  spellingLayout->addLayout (spellingDiacriticLayout);
  spellingLayout->addLayout (spellingCharacterBeforeLayout);
  spellingLayout->addLayout (spellingCharacterFollowingLayout);
  spellingLayout->addWidget (spellingCharacterDoubledButton);
  spellingLayout->addWidget (updateSpellingButton);
  spellingLayout->setAlignment (updateSpellingButton, Qt::AlignCenter);
  spellingGroupBox->setLayout (spellingLayout);

  representationDiacriticButton = new QRadioButton
                                          ("Add Diacritic (a e i o u y only):");
  representationDiacriticButton->setMinimumSize
                                   (representationDiacriticButton->sizeHint ());
  representationDiacriticButton->setChecked (true);
  representationDiacriticBox = new QComboBox;
  representationDiacriticBox->addItem ("Acute Accent");
  representationDiacriticBox->addItem ("Grave Accent");
  representationDiacriticBox->addItem ("Circumflex");
  representationDiacriticBox->addItem ("Diaresis/Umlaut");
  representationDiacriticBox->addItem ("Macron");
  representationDiacriticLayout = new QHBoxLayout;
  representationDiacriticLayout->addWidget (representationDiacriticButton);
  representationDiacriticLayout->addWidget (representationDiacriticBox);

  representationCharacterBeforeButton = new QRadioButton ("Character before:");
  representationCharacterBeforeEdit = new QLineEdit;
  representationCharacterBeforeEdit->setEnabled (false);
  representationCharacterBeforeLayout = new QHBoxLayout;
  representationCharacterBeforeLayout->addWidget
                                          (representationCharacterBeforeButton);
  representationCharacterBeforeLayout->addWidget
                                            (representationCharacterBeforeEdit);

  representationCharacterFollowingButton = new QRadioButton
                                                       ("Character following:");
  representationCharacterFollowingEdit = new QLineEdit;
  representationCharacterFollowingEdit->setEnabled (false);
  representationCharacterFollowingLayout = new QHBoxLayout;
  representationCharacterFollowingLayout->addWidget
                                       (representationCharacterFollowingButton);
  representationCharacterFollowingLayout->addWidget
                                         (representationCharacterFollowingEdit);

  representationCharacterDoubledButton = new QRadioButton ("Character doubled");
  updateRepresentationButton = new QPushButton ("Update");
  updateRepresentationButton->setMaximumSize
                                    (updateRepresentationButton->sizeHint ());
  updateRepresentationButton->setEnabled (false);
  representationGroupBox = new QGroupBox ("Phonemic Representation");
  representationLayout = new QVBoxLayout;
  representationLayout->addLayout (representationDiacriticLayout);
  representationLayout->addLayout (representationCharacterBeforeLayout);
  representationLayout->addLayout (representationCharacterFollowingLayout);
  representationLayout->addWidget (representationCharacterDoubledButton);
  representationLayout->addWidget (updateRepresentationButton);
  representationLayout->setAlignment
                                (updateRepresentationButton, Qt::AlignCenter);
  representationGroupBox->setLayout (representationLayout);

  QLabel *descriptionLabel = new QLabel ("Description:");
  descriptionEdit = new QTextEdit;
  descriptionEdit->setEnabled (false);

  infoLayout = new QVBoxLayout;
  infoLayout->addLayout (nameLayout);
  infoLayout->addLayout (domainLayout);
  infoLayout->addLayout (phonemesLayout);
  infoLayout->addWidget (spellingGroupBox);
  infoLayout->addWidget (representationGroupBox);
  infoLayout->addWidget (descriptionLabel);
  infoLayout->addWidget (descriptionEdit);

  mainLayout = new QHBoxLayout;
  mainLayout->addLayout (suprasegmentalListLayout);
  mainLayout->addLayout (moveUpDownLayout);
  mainLayout->addLayout (infoLayout);

  setLayout (mainLayout);

  connect (addSuprasegmentalButton, SIGNAL (clicked ()), this,
           SLOT (addSuprasegmental ()));
  connect (addSuprasegmentalButton, SIGNAL (clicked ()), addSuprasegmentalEdit,
           SLOT (clear ()));
  connect (deleteButton, SIGNAL (clicked ()), this,
           SLOT (deleteSuprasegmental ()));
  connect (editNameButton, SIGNAL (clicked ()), this, SLOT (editName ()));
  connect (phonApplyButton, SIGNAL (toggled (bool)), this, SLOT (changeDomain ()));
  connect (syllApplyButton, SIGNAL (toggled (bool)), this, SLOT (changeDomain ()));
  connect (editPhonemesButton, SIGNAL (clicked ()), this,
           SLOT (launchPhonemesDialog ()));
  connect (descriptionEdit, SIGNAL (textChanged ()), this,
           SLOT (editDescription ()));
  connect (updateSpellingButton, SIGNAL (clicked ()), this,
           SLOT (editSpelling ()));
  connect (updateRepresentationButton, SIGNAL (clicked ()), this,
           SLOT (editRepresentation ()));
  connect (moveUpButton, SIGNAL (clicked ()), this, SLOT (moveUp ()));
  connect (moveDownButton, SIGNAL (clicked ()), this, SLOT (moveDown ()));

  connect (spellingDiacriticButton, SIGNAL (toggled (bool)),
           spellingDiacriticBox, SLOT (setEnabled (bool)));
  connect (spellingCharacterBeforeButton, SIGNAL (toggled (bool)),
           spellingCharacterBeforeEdit, SLOT (setEnabled (bool)));
  connect (spellingCharacterFollowingButton, SIGNAL (toggled (bool)),
           spellingCharacterFollowingEdit, SLOT (setEnabled (bool)));
  connect (representationDiacriticButton, SIGNAL (toggled (bool)),
           representationDiacriticBox, SLOT (setEnabled (bool)));
  connect (representationCharacterBeforeButton, SIGNAL (toggled (bool)),
           representationCharacterBeforeEdit, SLOT (setEnabled (bool)));
  connect (representationCharacterFollowingButton, SIGNAL (toggled (bool)),
           representationCharacterFollowingEdit, SLOT (setEnabled (bool)));
}

void SuprasegmentalsPage::setModel (QStringListModel *model)  {
  suprasegmentalModel = model;
  suprasegmentalListView->setModel (model);

  connect (suprasegmentalListView->selectionModel (),
           SIGNAL (currentChanged (const QModelIndex&, const QModelIndex&)),
           this, SLOT (displaySuprasegmental ()));
  connect (suprasegmentalListView->selectionModel (),
           SIGNAL (selectionChanged (const QItemSelection&, const QItemSelection&)),
           this, SLOT (updateGUI ()));
}

void SuprasegmentalsPage::setList (QList< QPointer<Suprasegmental> > list)  {
  suprasegmentalList = list;
  updateModel ();
}

QList< QPointer<Suprasegmental> > SuprasegmentalsPage::getSupraList ()  {
  return suprasegmentalList;
}

void SuprasegmentalsPage::setPhonemeList (QList< QPointer<Phoneme> > list)  {
  phonemeList = list;

  for (int x = 0; x < suprasegmentalList.size (); x++)
    suprasegmentalList[x]->removeDuds ();
}

void SuprasegmentalsPage::setPhonemeFont (QFont font)  {
  applicablePhonemesLabel->setFont (font);
}

void SuprasegmentalsPage::setOrthographyFont (QFont font)  {
  spellingCharacterBeforeEdit->setFont (font);
  spellingCharacterFollowingEdit->setFont (font);
  representationCharacterBeforeEdit->setFont (font);
  representationCharacterFollowingEdit->setFont (font);
}

void SuprasegmentalsPage::setNotesFont (QFont font)  {
  addSuprasegmentalEdit->setFont (font);
  suprasegmentalListView->setFont (font);
  nameEdit->setFont (font);
  descriptionEdit->setFont (font);
}

void SuprasegmentalsPage::clear ()  {
  suprasegmentalList.clear ();
  updateModel ();
}

void SuprasegmentalsPage::addSuprasegmental ()  {
  if (addSuprasegmentalEdit->text () == "") return;

  int supraIndex = suprasegmentalList.size ();
  suprasegmentalList.append (new Suprasegmental);
  suprasegmentalList[suprasegmentalList.size () - 1]->setName
                                              (addSuprasegmentalEdit->text ());
  updateModel ();
  emit suprasegmentalListUpdated (suprasegmentalList);

  suprasegmentalListView->selectionModel ()->setCurrentIndex
  (suprasegmentalModel->index (supraIndex), QItemSelectionModel::SelectCurrent);
}

void SuprasegmentalsPage::deleteSuprasegmental ()  {
  QModelIndexList indexList = suprasegmentalListView->selectionModel ()->selectedRows ();

  for (int x = 0; x < indexList.size () - 1; x++)  {
    int largest = x;

    for (int y = x + 1; y < indexList.size (); y++)
      if (indexList[y].row () > indexList[largest].row ())
        largest = y;

    if (largest != x)
      indexList.swap (largest, x);
  }

  for (int x = 0; x < indexList.size (); x++)  {
    QPointer<Suprasegmental> s = suprasegmentalList[indexList[x].row ()];
    suprasegmentalList.removeAt (indexList[x].row ());
    delete s;
  }

  emit suprasegmentalListUpdated (suprasegmentalList);
  updateModel ();
}

void SuprasegmentalsPage::displaySuprasegmental ()  {
  int supraNum =
            suprasegmentalListView->selectionModel ()->currentIndex ().row ();
  if (supraNum < 0 || supraNum >= suprasegmentalList.size ()) return;

  updateGUI ();

  editNameButton->setEnabled (true);
  phonApplyButton->setEnabled (true);
  syllApplyButton->setEnabled (true);
  if (suprasegmentalList[supraNum]->getDomain () == PHON_APPL)
    editPhonemesButton->setEnabled (true);
  else editPhonemesButton->setEnabled (false);
  spellingDiacriticBox->setCurrentIndex (0);
  spellingCharacterBeforeEdit->clear ();
  spellingCharacterFollowingEdit->clear ();
  updateSpellingButton->setEnabled (true);
  representationDiacriticBox->setCurrentIndex (0);
  representationCharacterBeforeEdit->clear ();
  representationCharacterFollowingEdit->clear ();
  updateRepresentationButton->setEnabled (true);
  descriptionEdit->setEnabled (true);

  nameEdit->setText (suprasegmentalList[supraNum]->getName ());
  applicablePhonemesLabel->setText
                           (suprasegmentalList[supraNum]->getPhonemesText ());

  if (suprasegmentalList[supraNum]->getDomain () == PHON_APPL)
    phonApplyButton->setChecked (true);
  else syllApplyButton->setChecked (true);

  switch (suprasegmentalList[supraNum]->getSpellingType ())  {
    case DIACRITIC:
      spellingDiacriticButton->setChecked (true);
      spellingDiacriticBox->setEnabled (true);
      spellingDiacriticBox->setCurrentIndex (spellingDiacriticBox->findText
                          (suprasegmentalList[supraNum]->getSpellingDetail ()));
      break;

    case BEFORE:
      spellingCharacterBeforeButton->setChecked (true);
      spellingCharacterBeforeEdit->setEnabled (true);
      spellingCharacterBeforeEdit->setText
                          (suprasegmentalList[supraNum]->getSpellingDetail ());
      break;

    case FOLLOWING:
      spellingCharacterFollowingButton->setChecked (true);
      spellingCharacterFollowingEdit->setEnabled (true);
      spellingCharacterFollowingEdit->setText
                          (suprasegmentalList[supraNum]->getSpellingDetail ());
      break;

    case DOUBLED:
      spellingCharacterDoubledButton->setChecked (true);
      break;

    default: break;
  }

  switch (suprasegmentalList[supraNum]->getRepresentationType ())  {
    case DIACRITIC:
      representationDiacriticButton->setChecked (true);
      representationDiacriticBox->setEnabled (true);
      representationDiacriticBox->setCurrentIndex
              (representationDiacriticBox->findText
                    (suprasegmentalList[supraNum]->getRepresentationDetail ()));
      break;

    case BEFORE:
      representationCharacterBeforeButton->setChecked (true);
      representationCharacterBeforeEdit->setEnabled (true);
      representationCharacterBeforeEdit->setText
                    (suprasegmentalList[supraNum]->getRepresentationDetail ());
      break;

    case FOLLOWING:
      representationCharacterFollowingButton->setChecked (true);
      representationCharacterFollowingEdit->setEnabled (true);
      representationCharacterFollowingEdit->setText
                    (suprasegmentalList[supraNum]->getRepresentationDetail ());
      break;

    case DOUBLED:
      representationCharacterDoubledButton->setChecked (true);
      break;

    default: break;
  }

  descriptionEdit->setText (suprasegmentalList[supraNum]->getDescription ());
}

void SuprasegmentalsPage::updateGUI ()  {
  QModelIndexList indexList = suprasegmentalListView->selectionModel ()->selectedRows ();

  if (indexList.size () == 0)  {
    moveUpButton->setEnabled (false);
    moveDownButton->setEnabled (false);
    deleteButton->setEnabled (false);
    return;
  }

  moveUpButton->setEnabled (true);
  moveDownButton->setEnabled (true);
  deleteButton->setEnabled (true);

  for (int x = 0; x < indexList.size (); x++)  {
    if (indexList[x].row () == 0)
      moveUpButton->setEnabled (false);
    if (indexList[x].row () == (phonemeList.size () - 1))
      moveDownButton->setEnabled (false);
  }
}

void SuprasegmentalsPage::editName ()  {
  int supraNum =
              suprasegmentalListView->selectionModel ()->currentIndex ().row ();
  if (supraNum < 0 || supraNum >= suprasegmentalList.size ()) return;

  if (nameEdit->text () == "") return;

  suprasegmentalList[supraNum]->setName (nameEdit->text ());
  updateModel ();
  emit suprasegmentalListUpdated (suprasegmentalList);

  suprasegmentalListView->selectionModel ()->setCurrentIndex
  (suprasegmentalModel->index (supraNum), QItemSelectionModel::SelectCurrent);
}

void SuprasegmentalsPage::changeDomain ()  {
  int supraNum =
              suprasegmentalListView->selectionModel ()->currentIndex ().row ();
  if (supraNum < 0 || supraNum >= suprasegmentalList.size ()) return;

  if (phonApplyButton->isChecked ())  {
    editPhonemesButton->setEnabled (true);
    suprasegmentalList[supraNum]->setDomain (PHON_APPL);
  }

  else  {
    applicablePhonemesLabel->clear ();
    editPhonemesButton->setEnabled (false);
    suprasegmentalList[supraNum]->setPhonemeList (QList< QPointer<Phoneme> > ());
    suprasegmentalList[supraNum]->setDomain (SYLL_APPL);
  }
}

void SuprasegmentalsPage::launchPhonemesDialog ()  {
  int supraNum =
              suprasegmentalListView->selectionModel ()->currentIndex ().row ();
  if (supraNum < 0 || supraNum >= suprasegmentalList.size ()) return;

  if (choosePhonemesDialog)
    delete choosePhonemesDialog;

  choosePhonemesDialog = new ChoosePhonemesDialog (phonemeList,
                              suprasegmentalList[supraNum]->getPhonemeList ());
  choosePhonemesDialog->setPhonemeFont (applicablePhonemesLabel->font ());

  connect (choosePhonemesDialog,
           SIGNAL (chosenPhonemes (QList< QPointer<Phoneme> >)), this,
           SLOT (editPhonemes (QList< QPointer<Phoneme> >)));

  choosePhonemesDialog->exec ();
}

void SuprasegmentalsPage::editPhonemes (QList< QPointer<Phoneme> > list)  {
  int supraNum =
              suprasegmentalListView->selectionModel ()->currentIndex ().row ();
  if (supraNum < 0 || supraNum >= suprasegmentalList.size ()) return;

  suprasegmentalList[supraNum]->setPhonemeList (list);
  displaySuprasegmental ();
  emit suprasegmentalListUpdated (suprasegmentalList);
}

void SuprasegmentalsPage::editSpelling ()  {
  int supraNum =
              suprasegmentalListView->selectionModel ()->currentIndex ().row ();
  if (supraNum < 0 || supraNum >= suprasegmentalList.size ()) return;

  if (spellingDiacriticButton->isChecked ())
    suprasegmentalList[supraNum]->setSpellingRules
                            (DIACRITIC, spellingDiacriticBox->currentText ());

  if (spellingCharacterBeforeButton->isChecked ())
    suprasegmentalList[supraNum]->setSpellingRules
                                (BEFORE, spellingCharacterBeforeEdit->text ());

  if (spellingCharacterFollowingButton->isChecked ())
    suprasegmentalList[supraNum]->setSpellingRules
                          (FOLLOWING, spellingCharacterFollowingEdit->text ());

  if (spellingCharacterDoubledButton->isChecked ())
    suprasegmentalList[supraNum]->setSpellingRules (DOUBLED, "");

  emit suprasegmentalListUpdated (suprasegmentalList);
}

void SuprasegmentalsPage::editRepresentation ()  {
  int supraNum =
              suprasegmentalListView->selectionModel ()->currentIndex ().row ();
  if (supraNum < 0 || supraNum >= suprasegmentalList.size ()) return;

  if (representationDiacriticButton->isChecked ())
    suprasegmentalList[supraNum]->setRepresentationRules
                        (DIACRITIC, representationDiacriticBox->currentText ());

  if (representationCharacterBeforeButton->isChecked ())
    suprasegmentalList[supraNum]->setRepresentationRules
                          (BEFORE, representationCharacterBeforeEdit->text ());

  if (representationCharacterFollowingButton->isChecked ())
    suprasegmentalList[supraNum]->setRepresentationRules
                    (FOLLOWING, representationCharacterFollowingEdit->text ());

  if (representationCharacterDoubledButton->isChecked ())
    suprasegmentalList[supraNum]->setRepresentationRules (DOUBLED, "");

  emit suprasegmentalListUpdated (suprasegmentalList);
}

void SuprasegmentalsPage::editDescription ()  {
  int supraNum =
              suprasegmentalListView->selectionModel ()->currentIndex ().row ();
  if (supraNum < 0 || supraNum >= suprasegmentalList.size ()) return;

  suprasegmentalList[supraNum]->setDescription (descriptionEdit->toPlainText());
}

void SuprasegmentalsPage::moveUp ()  {
  QModelIndexList indexList = suprasegmentalListView->selectionModel ()->selectedRows ();
  QList<int> rowList;
  for (int x = 0; x < indexList.size (); x++)
    rowList.append (indexList[x].row ());

  for (int x = 0; x < rowList.size (); x++)
    suprasegmentalList.swap (rowList[x], rowList[x] - 1);
  emit suprasegmentalListUpdated (suprasegmentalList);

  updateModel ();

  for (int x = 0; x < rowList.size (); x++)
    suprasegmentalListView->selectionModel ()->select
      (suprasegmentalModel->index (rowList[x] - 1), QItemSelectionModel::Toggle);
}

void SuprasegmentalsPage::moveDown ()  {
  QModelIndexList indexList = suprasegmentalListView->selectionModel ()->selectedRows ();
  QList<int> rowList;
  for (int x = 0; x < indexList.size (); x++)
    rowList.append (indexList[x].row ());

  for (int x = 0; x < rowList.size (); x++)
    suprasegmentalList.swap (rowList[x], rowList[x] + 1);
  emit suprasegmentalListUpdated (suprasegmentalList);

  updateModel ();

  for (int x = 0; x < rowList.size (); x++)
    suprasegmentalListView->selectionModel ()->select
      (suprasegmentalModel->index (rowList[x] + 1), QItemSelectionModel::Toggle);
}

void SuprasegmentalsPage::updateModel ()  {
  suprasegmentalListView->selectionModel ()->reset ();
  moveUpButton->setEnabled (false);
  moveDownButton->setEnabled (false);
  deleteButton->setEnabled (false);
  editNameButton->setEnabled (false);
  nameEdit->clear ();
  editPhonemesButton->setEnabled (false);
  spellingDiacriticBox->setCurrentIndex (0);
  spellingCharacterBeforeEdit->clear ();
  spellingCharacterFollowingEdit->clear ();
  representationDiacriticBox->setCurrentIndex (0);
  representationCharacterBeforeEdit->clear ();
  representationCharacterFollowingEdit->clear ();
  updateSpellingButton->setEnabled (false);
  updateRepresentationButton->setEnabled (false);
  descriptionEdit->clear ();
  descriptionEdit->setEnabled (false);

  QStringList suprasegmentalNames;

  for (int x = 0; x < suprasegmentalList.size (); x++)
    suprasegmentalNames.append (suprasegmentalList[x]->getName ());

  suprasegmentalModel->setStringList (suprasegmentalNames);
}