#ifndef SUPRASEGMENTALSPAGE_H
#define SUPRASEGMENTALSPAGE_H

#include <QWidget>
#include <QList>
#include <QPointer>

class Suprasegmental;
class Phoneme;
class ChoosePhonemesDialog;
class QStringListModel;
class QListView;
class QPushButton;
class QLineEdit;
class QGroupBox;
class QRadioButton;
class QComboBox;
class QTextEdit;
class QLabel;
class QVBoxLayout;
class QHBoxLayout;

class SuprasegmentalsPage : public QWidget {
  Q_OBJECT

  public:
    SuprasegmentalsPage ();
    void setModel (QStringListModel*);
    void setList (QList< QPointer<Suprasegmental> >);
    QList< QPointer<Suprasegmental> > getSupraList ();

  signals:
    void suprasegmentalListUpdated (QList< QPointer<Suprasegmental> >);

  public slots:
    void setPhonemeList (QList< QPointer<Phoneme> >);
    void setPhonemeFont (QFont);
    void setOrthographyFont (QFont);
    void setNotesFont (QFont);
    void clear ();

  private slots:
    void addSuprasegmental ();
    void deleteSuprasegmental ();
    void displaySuprasegmental ();
    void updateGUI ();
    void editName ();
    void changeDomain ();
    void launchPhonemesDialog ();
    void editPhonemes (QList< QPointer<Phoneme> >);
    void editSpelling ();
    void editRepresentation ();
    void editDescription ();

    void moveUp ();
    void moveDown ();

  private:
    void updateModel ();
    
    QList< QPointer<Suprasegmental> > suprasegmentalList;
    QList< QPointer<Phoneme> > phonemeList;

    ChoosePhonemesDialog *choosePhonemesDialog;

    QStringListModel *suprasegmentalModel;

    QPushButton *addSuprasegmentalButton;
    QLineEdit *addSuprasegmentalEdit;
    QListView *suprasegmentalListView;
    QPushButton *moveUpButton;
    QPushButton *moveDownButton;
    QPushButton *deleteButton;

    QLineEdit *nameEdit;
    QPushButton *editNameButton;
    QRadioButton *phonApplyButton;
    QRadioButton *syllApplyButton;
    QLabel *applicablePhonemesLabel;
    QPushButton *editPhonemesButton;
    
    QGroupBox *spellingGroupBox;
    QRadioButton *spellingDiacriticButton;
    QRadioButton *spellingCharacterBeforeButton;
    QRadioButton *spellingCharacterFollowingButton;
    QRadioButton *spellingCharacterDoubledButton;
    QComboBox *spellingDiacriticBox;
    QLineEdit *spellingCharacterBeforeEdit;
    QLineEdit *spellingCharacterFollowingEdit;
    QPushButton *updateSpellingButton;
    
    QGroupBox *representationGroupBox;
    QRadioButton *representationDiacriticButton;
    QRadioButton *representationCharacterBeforeButton;
    QRadioButton *representationCharacterFollowingButton;
    QRadioButton *representationCharacterDoubledButton;
    QComboBox *representationDiacriticBox;
    QLineEdit *representationCharacterBeforeEdit;
    QLineEdit *representationCharacterFollowingEdit;
    QPushButton *updateRepresentationButton;

    QTextEdit *descriptionEdit;

    QHBoxLayout *addSuprasegmentalLayout;
    QVBoxLayout *suprasegmentalListLayout;
    QVBoxLayout *moveUpDownLayout;
    QHBoxLayout *nameLayout;
    QHBoxLayout *domainLayout;
    QHBoxLayout *phonemesLayout;
    QHBoxLayout *spellingDiacriticLayout;
    QHBoxLayout *spellingCharacterBeforeLayout;
    QHBoxLayout *spellingCharacterFollowingLayout;
    QVBoxLayout *spellingLayout;
    QHBoxLayout *representationDiacriticLayout;
    QHBoxLayout *representationCharacterBeforeLayout;
    QHBoxLayout *representationCharacterFollowingLayout;
    QVBoxLayout *representationLayout;
    QVBoxLayout *infoLayout;
    QHBoxLayout *mainLayout;
};

#endif