#ifndef SYLLABLE_H
#define SYLLABLE_H

#include <QList>
#include <QPointer>
#include <QObject>

#include "suprasegmental.h"

class Phoneme;
class QString;
class QStringList;

class Syllable {
  public:
    Syllable ();

    QList< QPointer<Phoneme> > getOnset ();
    QList< QPointer<Phoneme> > getPeak ();
    QList< QPointer<Phoneme> > getCoda ();
    QList< QList< QPointer<Suprasegmental> > > getOnsetSupras ();
    QList< QList< QPointer<Suprasegmental> > > getPeakSupras ();
    QList< QList< QPointer<Suprasegmental> > > getCodaSupras ();
    QStringList getOnsetSpellings ();
    QStringList getPeakSpellings ();
    QStringList getCodaSpellings ();
    QList< QPointer<Suprasegmental> > getSupras ();
    QPointer<Suprasegmental> getSupraBySpelling (int);
    QPointer<Suprasegmental> getSupraByRep (int);
    bool hasSupra (QPointer<Suprasegmental>);
    bool onsetHasSupra (int, QPointer<Suprasegmental>);
    bool peakHasSupra (int, QPointer<Suprasegmental>);
    bool codaHasSupra (int, QPointer<Suprasegmental>);
    QString getText ();
    QString getSaveOutput ();
    
    QList< QPointer<Suprasegmental> > popFirst (QPointer<Phoneme>&);
    QList< QPointer<Suprasegmental> > popLast (QPointer<Phoneme>&);
    void appendToOnset (QPointer<Phoneme>, QList< QPointer<Suprasegmental> >);
    void appendToCoda (QPointer<Phoneme>, QList< QPointer<Suprasegmental> >);

    void setOnset (QList< QPointer<Phoneme> >,
                   QList< QList< QPointer<Suprasegmental> > > =
                   QList< QList< QPointer<Suprasegmental> > > ());
    void setPeak (QList< QPointer<Phoneme> >,
                  QList< QList< QPointer<Suprasegmental> > > =
                  QList< QList< QPointer<Suprasegmental> > > ());
    void setCoda (QList< QPointer<Phoneme> >,
                  QList< QList< QPointer<Suprasegmental> > > =
                  QList< QList< QPointer<Suprasegmental> > > ());
    void setOnsetSpellings (QStringList);
    void setPeakSpellings (QStringList);
    void setCodaSpellings (QStringList);
    void clearSpellings ();
    void addSupra (QPointer<Suprasegmental>);
    void removeSupra (QPointer<Suprasegmental>);
    void clearSupraOfType (int);
    void removeDuds ();

    bool isEmpty ();

  private:
    QString applySupras (QPointer<Phoneme>, QList< QPointer<Suprasegmental> >);
    QString addDiacritic (QString, QString);

    QList< QPointer<Phoneme> > onset;
    QList< QPointer<Phoneme> > peak;
    QList< QPointer<Phoneme> > coda;
    QStringList onsetSpellings;
    QStringList peakSpellings;
    QStringList codaSpellings;
    QPointer<Suprasegmental> peakSupra;
    QPointer<Suprasegmental> beforeSupra;
    QPointer<Suprasegmental> followingSupra;

    QList< QList< QPointer<Suprasegmental> > > onsetSupras;
    QList< QList< QPointer<Suprasegmental> > > peakSupras;
    QList< QList< QPointer<Suprasegmental> > > codaSupras;
};

#endif