#include <QString>
#include <QTextStream>
#include <QMessageBox>

#include <iostream>
using namespace std;

#include "word.h"
#include "phoneme.h"
#include "suprasegmental.h"
#include "syllable.h"
#include "phonotactics.h"
#include "const.h"

Word::Word (QString n, Phonotactics phonotactics)  {
  name = n;
  type = "";
  subType = "";
  definition = "";

  calculatePhonology (phonotactics);
}

QString Word::getName ()  {
  return name;
}

QString Word::getPhonology ()  {
  QString text = "";

  for (int x = 0; x < phonology.size (); x++)
    text += phonology[x].getText () + ".";

  text.chop (1);

  if (text == "")
    return "???";

  return text;
}

QList<Syllable> Word::getSyllableList ()  {
  return phonology;
}

QString Word::getType ()  {
  return type;
}

QString Word::getSubType ()  {
  return subType;
}

QString Word::getDefinition ()  {
  return definition;
}

QString Word::getSaveOutput ()  {
  QString text = "";

  text += "\"" + name.replace ("\"", "\\\"") + "\",";
  text += "\"" + type.replace ("\"", "\\\"") + "\",";
  text += "\"" + subType.replace ("\"", "\\\"") + "\",";
  text += definition.replace (QRegExp ("\r?\n"), ";;");
  text += "\r\n";

  for (int x = 0; x < phonology.size (); x++)
    text += phonology[x].getSaveOutput () + "\r\n";

  text += "\r\n";

  return text;
}

void Word::setName (QString n, Phonotactics phonotactics, bool recalc) {
  name = n;
  if (recalc) calculatePhonology (phonotactics);
}

void Word::setPhonology (QList<Syllable> list)  {
  phonology = list;
}

void Word::setType (QString t)  {
  type = t;
}

void Word::setSubType (QString st)  {
  subType = st;
}

void Word::setDefinition (QString d)  {
  definition = d;
}

void Word::removeDuds ()  {
  for (int x = 0; x < phonology.size (); x++)
    phonology[x].removeDuds ();
}

bool Word::comesBefore (QPointer<Word> other, QList< QPointer<Phoneme> > alph){
  QList< QPointer<Phoneme> > thisPhonology;
  for (int x = 0; x < phonology.size (); x++)  {
    thisPhonology += phonology[x].getOnset ();
    thisPhonology += phonology[x].getPeak ();
    thisPhonology += phonology[x].getCoda ();
  }

  QList< QPointer<Phoneme> > otherPhonology;
  for (int x = 0; x < other->phonology.size (); x++)  {
    otherPhonology += other->phonology[x].getOnset ();
    otherPhonology += other->phonology[x].getPeak ();
    otherPhonology += other->phonology[x].getCoda ();
  }

  for (int x = 0; x < thisPhonology.size (); x++)  {
    if (x >= otherPhonology.size ()) return false;

    if (alph.indexOf (thisPhonology[x]) < alph.indexOf (otherPhonology[x]))
      return true;
    if (alph.indexOf (thisPhonology[x]) > alph.indexOf (otherPhonology[x]))
      return false;
  }

  if (otherPhonology.size () > thisPhonology.size ())
    return true;
  else return false;
}

void Word::calculatePhonology (Phonotactics phonotactics)  {
  phonology = phonotactics.parseWord (name);

  if (phonology.last ().isEmpty ()) phonology.removeLast ();

  for (int x = 0; x < phonology.size (); x++)
    phonology[x].clearSpellings ();
}