#ifndef WORDPAGE_H
#define WORDPAGE_H

#include <QWidget>
#include <QList>
#include <QPointer>

#include "phonotactics.h"
#include "syllable.h"

class Phoneme;
class Suprasegmental;
class Word;
class EditPhonologyDialog;
class QStringListModel;
class QLineEdit;
class QPushButton;
class QListView;
class QLabel;
class QCheckBox;
class QComboBox;
class QTextEdit;
class QVBoxLayout;
class QHBoxLayout;

class WordPage : public QWidget  {
  Q_OBJECT

  public:
    WordPage ();
    void setModel (QStringListModel*);
    QList< QPointer<Word> > getList ();
    Phonotactics getPhonotactics ();
    void setWordList (QList< QPointer<Word> >);
    bool containsWord (QString);
    void removeAll (QString);

  signals:
    void wordListUpdated (QList< QPointer<Word> >);

  public slots:
    void setPhonemeList (QList< QPointer<Phoneme> >);
    void setAlphabetOrder (QList< QPointer<Phoneme> >);
    void setSuprasegmentalList (QList< QPointer<Suprasegmental> >);
    void setPhonotactics (Phonotactics, bool = false);
    void setPhonemeFont (QFont);
    void setOrthographyFont (QFont);
    void setNotesFont (QFont);
    void setSquareBrackets (bool);
    void clear ();
    void addWords (QList< QPointer<Word> >, bool = false);

  private slots:
    void addWord ();
    void deleteWord ();
    void displayWord ();
    void editWordName ();
    void launchEditPhonologyDialog ();
    void editPhonology (QList<Syllable>);
    void editTypes ();
    void editDefinition ();
    void refreshTypeList ();
    void refreshSubTypeList ();

  private:
    void updateModel ();
    void sortWordList ();

    Phonotactics phonotactics;
    QList< QPointer<Phoneme> > alphabetOrder;
    QList< QPointer<Word> > wordList;
    bool squareBrackets;

    QStringListModel *wordListModel;

    EditPhonologyDialog *editPhonologyDialog;

    QLineEdit *addWordEdit;
    QPushButton *addWordButton;
    QPushButton *deleteButton;
    QListView *wordListView;

    QLineEdit *wordNameEdit;
    QPushButton *wordNameButton;
    QCheckBox *recalcPhonologyBox;
    QLabel *phonologyLabel;
    QPushButton *editPhonologyButton;
    QComboBox *typeBox;
    QComboBox *subTypeBox;
    QPushButton *editTypeButton;
    QTextEdit *definitionEdit;

    QHBoxLayout *addWordLayout;
    QVBoxLayout *wordListLayout;

    QHBoxLayout *wordNameLayout;
    QHBoxLayout *phonologyLayout;
    QHBoxLayout *typeLayout;
    QHBoxLayout *subTypeLayout;
    QVBoxLayout *typeBoxesLayout;
    QHBoxLayout *wordTypeLayout;
    QVBoxLayout *wordInfoLayout;

    QHBoxLayout *mainLayout;
};

#endif